//#include <QtGui>
#include "toma.h"
#include "strings.h"

const int heightOfRows = 22;		// Tablesize: height of rows
bool processingFlag = false;
bool fileTournamentSuffixEnabled = false; 

bool gotNewVersion = false;

quint8 currentTab = 0;

ToMa::ToMa()
{
  _appName = QFileInfo(QApplication::arguments().at(0)).baseName();
  preferredLanguage = QLocale::system().name().split('_')[0];
  QApplication::installTranslator(&translator);

  zdownloader = new ZDownloader(0, 0, false);
  zdownloader->setTarget(ZDownloader::ToMa);
  connect(zdownloader, SIGNAL(gotVersion(QString)),
          this, SLOT(zdownloaderGotVersion(QString)));
  zdownloader->downloadVersions();

  setApplicationData();
  reloadApplicationData();

  tabPosReached = 0;
  tbwDock = new QTabWidget(this); 

  matchInfo = new ZMatchInfo; 
  matchInfo->lblCountText = new QLabel(tr("Count: "));
  matchInfo->lblCount = new QLabel("0");
  matchInfo->lblPresentText = new QLabel(tr("Present: "));
  matchInfo->lblPresent = new QLabel("0");
  matchInfo->lblGroupMaxText = new QLabel(tr("Group max: "));
  matchInfo->lblLeagueText = new QLabel(tr("--- Leagues ---"));
  matchInfo->spbTeamMax = new QSpinBox();
  matchInfo->spbTeamMax->setRange(1, 99);
  matchInfo->spbTeamMax->setValue(splitTeamMax);
  matchInfo->spbTeamMax->setMaximumWidth(44);
  matchInfo->lblLeaguesText.resize(leaguesMax);
  matchInfo->lblLeagues.resize(leaguesMax);
  for(int i = 0; i < leaguesMax; i++) {
    matchInfo->lblLeaguesText[i] = new QLabel(tr("%1. league: ").arg(i + 1));
    matchInfo->lblLeagues[i] = new QLabel("0");
  }

  QFormLayout *frmInfo = new QFormLayout();
  frmInfo->setLabelAlignment(Qt::AlignLeft);
  frmInfo->addRow(matchInfo->lblCountText, matchInfo->lblCount);
  frmInfo->addRow(matchInfo->lblPresentText, matchInfo->lblPresent);
  frmInfo->addRow(matchInfo->lblGroupMaxText, matchInfo->spbTeamMax);
  frmInfo->setWidget(frmInfo->rowCount(), QFormLayout::SpanningRole, matchInfo->lblLeagueText);
  frmInfo->itemAt(frmInfo->rowCount() - 1, QFormLayout::SpanningRole)->setAlignment(Qt::AlignHCenter);

  for(int i = 0; i < leaguesMax; i++) {
    frmInfo->addRow(matchInfo->lblLeaguesText[i], matchInfo->lblLeagues[i]);
  }


  model = new QStandardItemModel(0, columnsList.count(), this);
  modelProxy = new QSortFilterProxyModel();
  modelProxy->setSourceModel(model);
  modelProxy->setSortLocaleAware(true);

  modelFinalOrder = new QStandardItemModel(0, columnsFinalOrderList.count(), this);

  tbvList = new QTableView(this);
  tbvList->setModel(model);
  tbvList->setShowGrid(true);
  tbvList->setAlternatingRowColors(true);
  tbvList->setSelectionMode(QAbstractItemView::SingleSelection);
  tbvList->setSelectionBehavior(QAbstractItemView::SelectRows);

  tbvFinalOrder = new QTableView(this);
  tbvFinalOrder->setShowGrid(true);
  tbvFinalOrder->setAlternatingRowColors(true);
  tbvFinalOrder->setModel(modelFinalOrder);
  tbvFinalOrder->setSelectionMode(QAbstractItemView::SingleSelection);
  tbvFinalOrder->setSelectionBehavior(QAbstractItemView::SelectRows);

  vblTeams = new QVBoxLayout;

  frmTeams = new QFrame();
  frmTeams->setLayout(vblTeams);

  slaTeams = new QScrollArea;
  slaTeams->setWidget(frmTeams);
  slaTeams->setWidgetResizable(true);
  slaTeams->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOn);
  slaTeams->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
  

  vblSpiders = new QVBoxLayout;

  frmSpiders = new QFrame();
  frmSpiders->setLayout(vblSpiders);

  slaSpiders = new QScrollArea;
  slaSpiders->setWidget(frmSpiders);
  slaSpiders->setWidgetResizable(true);
  slaSpiders->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOn);
  slaSpiders->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
  

  QHBoxLayout *hblList = new QHBoxLayout();
  hblList->addWidget(tbvList);
  hblList->addLayout(frmInfo);

  QFrame *frmList = new QFrame();
  frmList->setLayout(hblList);
  
  tbwDock->addTab(frmList, tabList.at(0));
  tbwDock->addTab(slaTeams, tabList.at(1));
  tbwDock->addTab(slaSpiders, tabList.at(2));
  tbwDock->addTab(tbvFinalOrder, tabList.at(3));

  setTabText();
  setHeaderData(); 
    
  for (int row = 0; row < model->rowCount(); ++row) {
    for (int column = 0; column < model->columnCount(); ++column) {
      model->setData(model->index(row, column, QModelIndex()), tr(""));
      model->setData(model->index(row, column, QModelIndex()), Qt::AlignVCenter, Qt::TextAlignmentRole);
    }
  }
  textResizeToContent();

  connect(statusBar(), SIGNAL(messageChanged(const QString &)),
          this, SLOT(statusBarMessageChanged(const QString &)));

  connect(tbwDock, SIGNAL(currentChanged(int)),
          this, SLOT(dockChanged(int)));
  connect(model, SIGNAL(itemChanged(QStandardItem*)),
          this, SLOT(itemChange(QStandardItem*)));


  connect(modelFinalOrder, SIGNAL(itemChanged(QStandardItem*)),
          tbvFinalOrder, SLOT(resizeRowsToContents()));
  connect(modelFinalOrder, SIGNAL(itemChanged(QStandardItem*)),
          tbvFinalOrder, SLOT(resizeColumnsToContents()));

  connect(matchInfo->spbTeamMax, SIGNAL(valueChanged(int)),
          this, SLOT(splitTeamMaxChange(int)));

  setCentralWidget(tbwDock);

  createActions();
  createMenus();
  createContextMenu();
  createToolBars();
  statusBarMessageChanged("");
  createTrayIcon();
  readSettings();

  setMinimumSize(400,470);
  resize(735,480);
  move((QApplication::desktop()->width() - width())/2,
       (QApplication::desktop()->height() - height())/2);

  if(recentFiles.count()) {
    if(!loadFile(recentFiles.at(0)))
      newFile();
  } else
    newFile();

}

//function emitted when application close
void ToMa::closeEvent(QCloseEvent *event)
{
  if(okToContinue()) {
    writeSettings();
    event->accept();
  } else {
    event->ignore();
  }
}

void ToMa::createActions()
{
  newAct = new QAction(tr("&New"), this);
  newAct->setIcon(QIcon(":/images/new.png"));
  newAct->setShortcut(tr("Ctrl+N"));
  newAct->setStatusTip(tr("Create a new file"));
  connect(newAct, SIGNAL(triggered()), this, SLOT(newFile()));

  openAct = new QAction(tr("&Open..."), this);
  openAct->setIcon(QIcon(":/images/open.png"));
  openAct->setShortcut(tr("Ctrl+O"));
  openAct->setStatusTip(tr("Open an existing file"));
  connect(openAct, SIGNAL(triggered()), this, SLOT(open()));
  
  saveAct = new QAction(tr("&Save"), this);
  saveAct->setIcon(QIcon(":/images/save.png"));
  saveAct->setShortcut(tr("Ctrl+S"));
  saveAct->setStatusTip(tr("Save the document to disk"));
  connect(saveAct, SIGNAL(triggered()), this, SLOT(save()));

  saveAsAct = new QAction(tr("Save &As"), this);
  saveAsAct->setIcon(QIcon(":/images/saveAs.png"));
  saveAsAct->setStatusTip(tr("Save the document under a new name"));
  connect(saveAsAct, SIGNAL(triggered()), this, SLOT(saveAs()));

  for(int i = 0; i < MaxRecentFiles; ++i) {
    recentFileActs[i] = new QAction(this);
    recentFileActs[i]->setVisible(false);
    connect(recentFileActs[i], SIGNAL(triggered()),
            this, SLOT(openRecentFile()));
  }

  exitAct = new QAction(tr("E&xit"), this);
  exitAct->setIcon(QIcon(":/images/exit.png"));
  exitAct->setShortcut(tr("Ctrl+Q"));
  exitAct->setStatusTip(tr("Exit the application"));
  connect(exitAct, SIGNAL(triggered()), this, SLOT(close()));

  addLineAct = new QAction(tr("A&dd Line"), this);
  addLineAct->setIcon(QIcon(":/images/list-add.png"));
  addLineAct->setShortcut(tr("Ctrl+Ins"));
  addLineAct->setStatusTip(tr("Add new line"));
  connect(addLineAct, SIGNAL(triggered()), this, SLOT(addLine()));

  delLineAct = new QAction(tr("D&elete Line"), this);
  delLineAct->setIcon(QIcon(":/images/list-remove.png"));
  delLineAct->setShortcut(tr("Ctrl+Del"));
  delLineAct->setStatusTip(tr("Delete selected line"));
  connect(delLineAct, SIGNAL(triggered()), this, SLOT(delLine()));
  
  sortAct = new QAction(tr("So&rt"), this);
  sortAct->setIcon(QIcon(":/images/sort_ascend.png"));
  sortAct->setShortcut(tr("Ctrl+R"));
  sortAct->setStatusTip(tr("Sort lines"));
  connect(sortAct, SIGNAL(triggered()), this, SLOT(sort()));

  sortRevAct = new QAction(tr("Sort Rever&t"), this);
  sortRevAct->setIcon(QIcon(":/images/sort_descend.png"));
  sortRevAct->setShortcut(tr("Ctrl+T"));
  sortRevAct->setStatusTip(tr("Sort lines revert"));
  connect(sortRevAct, SIGNAL(triggered()), this, SLOT(sort_rev()));

  playAct = new QAction(tr("&Play"), this);
  playAct->setIcon(QIcon(":/images/go-jump.png"));
  playAct->setShortcut(tr("Ctrl+P"));
  playAct->setStatusTip(tr("Play tournament"));
  connect(playAct, SIGNAL(triggered()), this, SLOT(play()));

  moveUpAct = new QAction(tr("Move &up"), this);
  moveUpAct->setShortcut(tr("Ctrl+U"));
  moveUpAct->setIcon(QIcon(":/images/go-up.png"));
  moveUpAct->setStatusTip(tr("Move row up"));
  connect(moveUpAct, SIGNAL(triggered()), this, SLOT(moveUp()));

  moveDownAct = new QAction(tr("Move &down"), this);
  moveDownAct->setIcon(QIcon(":/images/go-down.png"));
  moveDownAct->setShortcut(tr("Ctrl+D"));
  moveDownAct->setStatusTip(tr("Move row down"));
  connect(moveDownAct, SIGNAL(triggered()), this, SLOT(moveDown()));

  moveToAct = new QAction(tr("Move &to"), this);
  moveToAct->setIcon(QIcon(":/images/refresh.png"));
  moveToAct->setShortcut(tr("Ctrl+T"));
  moveToAct->setStatusTip(tr("Move row to"));
  connect(moveToAct, SIGNAL(triggered()), this, SLOT(moveTo()));


  updateAct = new QAction(tr("&Update application"), this);
  updateAct->setStatusTip(tr("Check new version and update application"));
  connect(updateAct, SIGNAL(triggered()), this, SLOT(update()));

  aboutAct = new QAction(tr("&About"), this);
  aboutAct->setStatusTip(tr("Show the application's About box"));
  connect(aboutAct, SIGNAL(triggered()), this, SLOT(about()));

    
  styleActGroup = new QActionGroup(this);
  connect(styleActGroup, SIGNAL(triggered(QAction*)),
          this, SLOT(changeStyle(QAction*)));
  charCodeActGroup = new QActionGroup(this);
  connect(charCodeActGroup, SIGNAL(triggered(QAction*)),
          this, SLOT(changeCharCode(QAction*)));
  languageActGroup = new QActionGroup(this);
  connect(languageActGroup, SIGNAL(triggered(QAction*)),
          this, SLOT(changeLanguage(QAction*)));

  minAct = new QAction(tr("Mi&nimize"), this);
  connect(minAct, SIGNAL(triggered()), this, SLOT(hide()));
  maxAct = new QAction(tr("Ma&ximize"), this);
  connect(maxAct, SIGNAL(triggered()), this, SLOT(showMaximized()));
  restAct = new QAction(tr("&Restore"), this);
  connect(restAct, SIGNAL(triggered()), this, SLOT(showNormal()));
}

void ToMa::createMenus()
{
  fileMenu = menuBar()->addMenu(tr("&File"));
  fileMenu->addAction(newAct);
  fileMenu->addAction(openAct);
  fileMenu->addAction(saveAct);
  fileMenu->addAction(saveAsAct);
  separatorAct = fileMenu->addSeparator();
  for (int i = 0; i < MaxRecentFiles; ++i)
      fileMenu->addAction(recentFileActs[i]);
  fileMenu->addSeparator();
  fileMenu->addAction(exitAct);
  updateRecentFileActions();

  editMenu = menuBar()->addMenu(tr("&Edit"));
  editMenu->addAction(addLineAct);
  editMenu->addAction(delLineAct);
  
  toolsMenu = menuBar()->addMenu(tr("&Tools"));
  toolsMenu->addAction(playAct);
  toolsMenu->addSeparator();
  toolsMenu->addAction(moveToAct);
  toolsMenu->addAction(moveUpAct);
  toolsMenu->addAction(moveDownAct);
  toolsMenu->addSeparator();
  toolsMenu->addAction(sortAct);
  toolsMenu->addAction(sortRevAct);

  optionsMenu = menuBar()->addMenu(tr("&Options"));

  charCodeSubMenu = optionsMenu->addMenu(tr("Character &Encoding"));

  foreach(QString charCode, charCodeList)
  {
    charCodeMap[charCode] = new QAction(charCode, this);
    charCodeMap[charCode]->setData(charCode);
    charCodeMap[charCode]->setCheckable(true);
    charCodeSubMenu->addAction(charCodeMap[charCode]);
    charCodeActGroup->addAction(charCodeMap[charCode]);
  }
  

  styleSubMenu = optionsMenu->addMenu(tr("&Style"));

  foreach(QString style, QStyleFactory::keys())
  {
    styleMap[style] = new QAction(style, this);
    styleMap[style]->setData(style);
    styleMap[style]->setCheckable(true);
    styleSubMenu->addAction(styleMap[style]);
    styleActGroup->addAction(styleMap[style]);
  }
 

  langSubMenu = optionsMenu->addMenu(tr("&Language"));
  
  foreach(QString lang, langShortMap)
  {
    languageMap[lang] = new QAction(lang, this);
    languageMap[lang]->setData(lang);
    languageMap[lang]->setCheckable(true);
    langSubMenu->addAction(languageMap[lang]);
    languageActGroup->addAction(languageMap[lang]);
  }


  helpMenu = menuBar()->addMenu(tr("&Help"));
  helpMenu->addAction(updateAct);
  helpMenu->addAction(aboutAct);
}

void ToMa::createContextMenu()
{
}

void ToMa::createToolBars()
{
  fileToolBar = addToolBar(tr("&File"));
  fileToolBar->addAction(newAct);
  fileToolBar->addAction(openAct);
  fileToolBar->addAction(saveAct);

  editToolBar = addToolBar(tr("&Edit"));
  editToolBar->addAction(playAct);
  editToolBar->addSeparator();
  editToolBar->addAction(moveToAct);
  editToolBar->addAction(moveUpAct);
  editToolBar->addAction(moveDownAct);
  editToolBar->addSeparator();
  editToolBar->addAction(sortAct);
  editToolBar->addAction(sortRevAct);
  editToolBar->addAction(addLineAct);
  editToolBar->addAction(delLineAct);

}

void ToMa::createTrayIcon()
{
  trayIconMenu = new QMenu(this);
  trayIconMenu->addAction(minAct);
  trayIconMenu->addAction(maxAct);
  trayIconMenu->addAction(restAct);
  trayIconMenu->addSeparator();
  trayIconMenu->addAction(exitAct);

  icon = QIcon(":/images/myappG.png");
  trayIcon = new QSystemTrayIcon(this);
  trayIcon->setContextMenu(trayIconMenu);
  trayIcon->setIcon(icon);
  setWindowIcon(icon);
  trayIcon->show();
  
  connect(trayIcon, SIGNAL(activated(QSystemTrayIcon::ActivationReason)),
          this, SLOT(trayIconActivated(QSystemTrayIcon::ActivationReason)));
}

void ToMa::statusBarMessageChanged(const QString &message)
{
  if(message == "")
    statusBar()->showMessage(tr("Copyright %1 2014-2015 Zakrava. All rights reserved.").arg(QChar(0x00A9)));
}

//run update 
void ToMa::update()
{
  if(!gotNewVersion) {
    QMessageBox::information(this, prog, tr("Application is up to date."));
  } else {
    if(QProcess::startDetached(QFileInfo(zdownloader->appName() + zdownloader->osSuffix()).absoluteFilePath())) {
      qApp->quit(); 
    } else {
      zdownloader->downloadOnce(zdownloader->appName());
      zdownloader->show();
    }
  }
}

//show about 
void ToMa::about()
{
   QMessageBox::about(this, tr("About %1").arg(prog),
     tr("This is <b>%1 v%2</b>.<br />Application created by <b>Zakrava</b> for FIM UHK.<br />Copyright &copy; 2014-2015 Zakrava. All rights reserved.").arg(prog).arg(progVer));
}

//show saving dialog
bool ToMa::okToContinue()
{
  if(isWindowModified()) {
    QMessageBox mbx(prog,
      tr("The data has been modified.\n"
         "Do you want to save changes?"),
      QMessageBox::Warning,
      QMessageBox::Yes | QMessageBox::Default,
      QMessageBox::No,
      QMessageBox::Cancel | QMessageBox::Escape);
    mbx.setButtonText( QMessageBox::Yes, tr("&Yes") );
    mbx.setButtonText( QMessageBox::No, tr("&No") );
    mbx.setButtonText( QMessageBox::Cancel, tr("&Cancel") );

    switch(mbx.exec()){
	    case QMessageBox::Yes:
            return true;
	        break;
      case QMessageBox::Cancel:
            return false;
	        break;
    }
  }
  return true;
}

//set current file
void ToMa::setCurrentFile(const QString &fileName)
{
  curFile = fileName;
  setWindowModified(false);
  QString shownName = tr("Untitled");

  if(!curFile.isEmpty()) {
    shownName = strippedName(curFile);
    recentFiles.removeAll(curFile);
    recentFiles.prepend(curFile);
    updateRecentFileActions();
  }

  if(shownName == tr("Untitled"))
    setWindowTitle(tr("%1[*] - %2").arg(shownName).arg(prog));
  else
    setWindowTitle(tr("%1[*] - %2").arg(shownName.left(shownName.length())).arg(prog));
}

//update recent files
void ToMa::updateRecentFileActions()
{
    QMutableStringListIterator i(recentFiles);
    while (i.hasNext()) {
        if (!QFile::exists(i.next()))
            i.remove();
    }
    for (int j = 0; j < MaxRecentFiles; ++j) {
        if (j < recentFiles.count()) {
            QString text = tr("&%1 %2")
                           .arg(j + 1)
                           .arg(strippedName(recentFiles[j]));
            recentFileActs[j]->setText(text);
            recentFileActs[j]->setData(recentFiles[j]);
            recentFileActs[j]->setVisible(true);
        } else {
            recentFileActs[j]->setVisible(false);
        }
    }
    separatorAct->setVisible(!recentFiles.isEmpty());
}

//cutting filename 
QString ToMa::strippedName(const QString &fullFileName)
{
  return QFileInfo(fullFileName).fileName();
}

void ToMa::clear()
{
  tabPosReached = 0;
  model->setRowCount(0);

  for(int x = 0; x < modelGroups.count(); x++) {
    delete modelGroups[x];
   }

  for(int x = 0; x < matchInfo->lblLeagues.count(); x++)
    matchInfo->lblLeagues[x]->setNum(0);
  playersUpdate();
  leaguesUpdate();
  
  leagues.clear();
  leaguesNumbers.clear();
  teamsGroups.clear();
  teamsMembers.clear();
  modelGroups.clear();

  clearTeamsTab();
  clearSpidersTab();
}

void ToMa::clearTeamsTab()
{
  for(int x = 0; x < tbvTeams.count(); x++) {
    vblTeams->removeWidget(tbvTeams[x]);
    vblTeams->removeWidget(lblTeams[x]);

    tbvTeams[x]->setModel(NULL);
    tbvTeams[x]->close();
    lblTeams[x]->close();

    delete modelTeams[x];
    delete tbvTeams[x];
    delete lblTeams[x];
  }

  if(tbvTeams.count()) {
    for(int x = vblTeams->count(); x > 0; x--)
      vblTeams->removeItem(vblTeams->itemAt(x));

    vblTeams->setContentsMargins(QMargins(9,0,9,9));
  }

  lblTeams.clear();
  modelTeams.clear();
  tbvTeams.clear();
  teamsPlayersModels.clear();
}

void ToMa::clearSpidersTab()
{
  for(int x = 0; x < wdtSpiders.count(); x++) {
    vblSpiders->removeWidget(wdtSpiders[x]);

    wdtSpiders[x]->close();

    delete wdtSpiders[x];
  }

  if(wdtSpiders.count()) {
    for(int x = vblSpiders->count(); x > 0; x--)
      vblSpiders->removeItem(vblSpiders->itemAt(x));

    vblSpiders->setContentsMargins(QMargins(9,0,9,9));
  }

  wdtSpiders.clear();
  playersGroups.clear();
  playersGroupsResults.clear();
}

//open recent file
void ToMa::openRecentFile()
{
  if(okToContinue()) {
    QAction *action = qobject_cast<QAction *>(sender());
    if(action)
      loadFile(action->data().toString());
  }
}

//new file
void ToMa::newFile()
{
  clear();
  setCurrentFile("");
  textResizeToContent();
  tbwDock->setCurrentIndex(tabList.indexOf(tr("Main")));
}

//open file
void ToMa::open()
{
  QString selectedFilter;
  QString fileName = QFileDialog::getOpenFileName(this,
        		        tr("Open file"), ".",
       				      tr("All (*.xls *.csv *.%2 *.txt);;M$ Office - Excel (*.xls);;Text CSV (*.csv);;%1 (*.%2);;Text (*.txt)").arg(fileTournamentSuffixLabel).arg(fileTournamentSuffix), &selectedFilter);

  
  if(!fileName.isEmpty())
    loadFile(fileName);
}

//load file
bool ToMa::loadFile(const QString &fileName)
{
  clear();
  QApplication::setOverrideCursor(Qt::WaitCursor);
  
  QFileInfo fileInfo(fileName);
  
  if(fileInfo.suffix() == fileTournamentSuffix || fileInfo.suffix() == "xls") {
    if(!fileInfo.isReadable()) {
      statusLoadFailedMessageRestoreCursor();
      return false;
    } else
      if(fileInfo.suffix() == "xls") {
        readXLSFileToModel(fileName); 
      } else
        readTournamentSettings(fileName);
  } else {
    if(!readFile(fileName)) {
      statusLoadFailedMessageRestoreCursor();
      return false;
    }
  }
  
  QApplication::restoreOverrideCursor();
    
  setCurrentFile(fileName);
  statusBar()->showMessage(tr("File loaded"), 2000);
  
  playersUpdate();
  leaguesUpdate();

  return true;
}

void ToMa::statusLoadFailedMessageRestoreCursor()
{
  statusBar()->showMessage(tr("Loading canceled"), 2000);
  QApplication::restoreOverrideCursor();
}

bool ToMa::readFile(const QString &fileName)
{
  processingFlag = true;
  QFile file(fileName);
   
  if(!file.open(QIODevice::ReadOnly | QIODevice::Text)) {
    return false;
  }

  QTextStream stream(&file);
  readToModel(stream, fileEncoding);
  
  file.close();
  processingFlag = false;
  textResizeToContent();

  return true;
}

bool ToMa::readXLSFileToModel(const QString &fileName)
{
  processingFlag = true;
  clear();
 
  size_t sheetIndex = 0;
	BasicExcel excel;

	if(!excel.Load(fileName.toLatin1()))
    return false;
  

	BasicExcelWorksheet *excelSheet = excel.GetWorksheet(sheetIndex);
	if(excelSheet) {
    int row = 0;
    int column;
    QString str;
 		size_t maxRows = excelSheet->GetTotalRows();
		size_t maxCols = excelSheet->GetTotalCols();

		for (size_t r = 0; r < maxRows; ++r) {
      column = 0;
			for (size_t c = 0; c < maxCols; ++c) {
				BasicExcelCell *cell = excelSheet->Cell(r,c);
				if((cell->Type() != BasicExcelCell::INT && cell->Type() != BasicExcelCell::DOUBLE && !c) ||  cell->Type() == BasicExcelCell::UNDEFINED)
          break;

        if(c == 0) {
          model->setRowCount(model->rowCount()+1);
          row = model->rowCount()-1;
          setCheckable(row);
          continue;
        }

        switch (cell->Type()) {
					case BasicExcelCell::INT:

					case BasicExcelCell::DOUBLE:
            str = c == 1 ? str.sprintf("%2d", cell->GetInteger()) : QString::number(cell->GetInteger());
            model->setData(model->index(row, column++, QModelIndex()), str);  
						break;

					case BasicExcelCell::STRING:
            str = tr("%0").arg(cell->GetString()).trimmed();
            if(str[str.length() - 1] == '.')
              str.remove(str.length() - 1, 1);
            if(c == 1 && str.toUInt())
              str = str.sprintf("%2d", str.toUInt());
            model->setData(model->index(row, column++, QModelIndex()), str);
						break;

					case BasicExcelCell::WSTRING:
            model->setData(model->index(row, column++, QModelIndex()), tr("%0").arg(QString::fromWCharArray(cell->GetWString())));  
						break;
				}
			}
		}
	}
  
  processingFlag = false;
  textResizeToContent();

  return true;
}

void ToMa::readToModel(QTextStream &stream, QString &streamEncoding)
{
  clear();
  stream.setCodec(QTextCodec::codecForName(streamEncoding.toAscii()));
  
  while(!stream.atEnd()) {
    QString str = stream.readLine();
    QStringList strlist = str.split(";");
    model->setRowCount(model->rowCount()+1);
    int row = model->rowCount()-1;
    int column = 0;
    setCheckable(row);
    foreach (QString buf, strlist) {
      if(!column && buf.count() == 1)
        buf.prepend(' ');
      model->setData(model->index(row, column++, QModelIndex()), tr("%0").arg(buf));  
    }
  }
}

//saving
bool ToMa::save()
{
  if(curFile.isEmpty()) {
    return saveAs();
  } else {   
    return saveFile(curFile);
  }
}

//save file as
bool ToMa::saveAs()
{
  QString selectedFilter;
  if(fileTournamentSuffixEnabled)
    selectedFilter = QString("%1 (*.%2)").arg(fileTournamentSuffixLabel).arg(fileTournamentSuffix);
  QString fileName = QFileDialog::getSaveFileName(this,
                          tr("Save file as"), ".",
                          tr("CSV file (*.csv);;%1 (*.%2);;Text (*.txt);;M$ Office - Excel (*.xls)").arg(fileTournamentSuffixLabel).arg(fileTournamentSuffix), &selectedFilter);

  if(fileName.isEmpty())
    return false;
  QString suffix = selectedFilter.right(5).left(4);
  if(!fileName.endsWith(suffix, Qt::CaseInsensitive))
    fileName.append(suffix);
  return saveFile(fileName);
}

//save file
bool ToMa::saveFile(const QString &fileName)
{
  QApplication::setOverrideCursor(Qt::WaitCursor);

  QFileInfo fileInfo(fileName);

  if(fileInfo.suffix() == fileTournamentSuffix) {
      writeTournamentSettings(fileName);
  } else if(fileInfo.suffix() == "xls") {
    writeXLSFile(fileName);
  } else {
    if(!writeFile(fileName)) {
      statusSaveFailedMessageRestoreCursor();
      return false;
    }
  }
 
  QApplication::restoreOverrideCursor();


  if(!curFile.isEmpty()) {
    QFileInfo curFileInfo = QFileInfo(curFile);
    if(curFileInfo.suffix() == fileTournamentSuffix && fileInfo.suffix() != fileTournamentSuffix) {
      goto skipSettingCurrentFile;
    }
  }

  setCurrentFile(fileName);
skipSettingCurrentFile:
  statusBar()->showMessage(tr("File saved"), 2000);
  return true;

}

void ToMa::statusSaveFailedMessageRestoreCursor()
{
  statusBar()->showMessage(tr("Saving canceled"), 2000);
  QApplication::restoreOverrideCursor();
}

bool ToMa::writeXLSFile(const QString &fileName)
{

  QStandardItemModel *modelTmp = currentTab == tabList.size() - 1 ? modelFinalOrder : model;

	BasicExcel excel;
  size_t sheetIndex = 0;

  excel.New(1);
	BasicExcelWorksheet  *excelSheet = excel.GetWorksheet(sheetIndex);

	if(excelSheet) {
    for(int row = 0; row < (modelTmp->rowCount()); ++row) {
      excelSheet->Cell(row, 0)->SetInteger(row + 1);
      for(int column = 0; column < modelTmp->columnCount(); ++column) {
        excelSheet->Cell(row, column + 1)->SetWString(modelTmp->data(modelTmp->index(row, column, QModelIndex()), Qt::DisplayRole).toString().toStdWString().c_str());
      }
    }
  }
	
  return excel.SaveAs(fileName.toLatin1());
}

bool ToMa::writeFile(const QString &fileName)
{
  QFile file(fileName);
  if(!file.open(QIODevice::WriteOnly | QIODevice::Text))
    return false;

  QTextStream out(&file);
  out.setCodec(QTextCodec::codecForName(fileEncoding.toAscii()));
  
  QStandardItemModel *modelTmp = currentTab == tabList.size() - 1 ? modelFinalOrder : model;
  for(int row = 0; row < (modelTmp->rowCount()); ++row) {
    out << row + 1 << ";";
    for(int column = 1; column < modelTmp->columnCount(); ++column) {
      out << modelTmp->data(modelTmp->index(row, column, QModelIndex()), Qt::DisplayRole).toString();
      if(column == (modelTmp->columnCount()-1))
        out << endl;
      else
        out << ";";
    }
  }
  file.close();
  return true;
}

void ToMa::play()
{
  if(currentTab < tabPosReached) {
    tbwDock->setCurrentIndex(++currentTab);
    return;
  }

  if(currentTab == tabList.indexOf(tr("Spiders"))) {
    if(!spiderFinalOrderMaker())
    QMessageBox::information(this, prog, tr("All spiders results must be filled!"));
    return;
  } else if(currentTab == tabList.indexOf(tr("Groups"))) {
    createSpidersTabData();
    createSpidersTab();
    return;
  }

  calcLeagues();
  teamsGroups.clear();
  for(QMap<int, int>::iterator iterator = leaguesNumbers.begin(); iterator != leaguesNumbers.end(); ++iterator ) {
    int league = iterator.key();
    int leagueCount = iterator.value();

    int numTeams = leagueCount / splitTeamMax;
    int lastTeam = leagueCount % splitTeamMax;
    if(lastTeam != 0) {
      numTeams++;
      int avgFloor = leagueCount / (numTeams);
      int rest = leagueCount - (avgFloor * numTeams);
      for(int i = 0; i < rest; i++)
        teamsGroups[league] << (avgFloor + 1);
      for(int i = 0; i < numTeams - rest; i++)
        teamsGroups[league] << (avgFloor);
    } else {
      for(int i = 0; i < numTeams; i++)
        teamsGroups[league] << (splitTeamMax);
    }
  }
  QMap<int, QMap<QString, QStringList> > teamsMembersFinal;
  teamsMembersFinal.clear();
  for(QMap<int, QStringList >::iterator iterator = teamsMembers.begin(); iterator != teamsMembers.end(); ++iterator ) {
    for(int i = 0; i < iterator.value().count(); i++) {
      QString teamName = QString("%2").arg((char) (charBigFirst + (i % teamsGroups[iterator.key()].count())));
      teamsMembersFinal[iterator.key()][teamName] << iterator.value()[i];
    }
  }

//Teams drag & drop dialog
  QDialog *dlgDnDTeams = new QDialog();
  
  QVBoxLayout *vblDnDTeams = new QVBoxLayout;
  
  QFrame *frmTeam = new QFrame();
  frmTeam->setLayout(vblDnDTeams);

  QScrollArea *slaTeam = new QScrollArea;
  slaTeam->setWidget(frmTeam);
  slaTeam->setWidgetResizable(true);
  slaTeam->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOn);
  slaTeam->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
 
  QPushButton *btnOk = new QPushButton(tr("&Ok"));
  QPushButton *btnCancel = new QPushButton(tr("&Cancel"));

  connect(btnOk, SIGNAL(clicked()),
          dlgDnDTeams, SLOT(accept()));
  connect(btnCancel, SIGNAL(clicked()),
          dlgDnDTeams, SLOT(reject()));

  QHBoxLayout *hblButtons = new QHBoxLayout;
  hblButtons->setContentsMargins(9, 4, 9, 9);
  hblButtons->addWidget(btnOk);
  hblButtons->addWidget(btnCancel);

  QVBoxLayout *vblSLATeam = new QVBoxLayout;
  vblSLATeam->setContentsMargins(0, 0, 0, 0);
  vblSLATeam->addWidget(slaTeam);
  vblSLATeam->addLayout(hblButtons);
  dlgDnDTeams->setLayout(vblSLATeam);

  int i = 0;
  for(int x = 0; x < modelGroups.count(); x++)
    delete modelGroups[x];
  modelGroups.clear();
  teamsNames.clear();
  for(QMap<int, QMap<QString, QStringList> >::iterator iterator = teamsMembersFinal.begin(); iterator != teamsMembersFinal.end(); ++iterator) {
    QHBoxLayout *hblDnDTeams = new QHBoxLayout;
    for(QMap<QString, QStringList>::iterator iterator2 = iterator.value().begin(); iterator2 != iterator.value().end(); ++iterator2) {
      teamsNames << QString(tr("Group: ") + QString("%1").arg(iterator.key()) + iterator2.key());
      QLabel *lblTeam = new QLabel(teamsNames[i]);
      QTableView *tbvTeam = new QTableView();
      modelGroups << new QStandardItemModel(0, 1, this);
      tbvTeam->setShowGrid(true);
      tbvTeam->setModel(modelGroups[i]);
      tbvTeam->setEditTriggers(QAbstractItemView::NoEditTriggers);
      tbvTeam->setSelectionMode(QAbstractItemView::SingleSelection);
      tbvTeam->setAlternatingRowColors(true);
      tbvTeam->horizontalHeader()->hide();

      tbvTeam->setAcceptDrops(true);
      tbvTeam->setDefaultDropAction(Qt::MoveAction);
      tbvTeam->setDragDropMode(QAbstractItemView::DragDrop);
      tbvTeam->setDragDropOverwriteMode(false);
      tbvTeam->setDragEnabled(true);
      tbvTeam->setDropIndicatorShown(true);

      modelGroups[i]->setRowCount(iterator2.value().count());
      for(int row = 0; row < iterator2.value().count(); row++) {
        modelGroups[i]->setData(modelGroups[i]->index(row, 0, QModelIndex()), iterator2.value()[row]);  
        modelGroups[i]->item(row, 0)->setDropEnabled(false);
      }  

      tbvTeam->resizeRowsToContents();
      tbvTeam->resizeColumnsToContents();

      int columnsWidth = tbvTeam->columnWidth(0) + tbvTeam->verticalHeader()->sectionSize(0);//size().width();
      tbvTeam->setMinimumWidth(columnsWidth);

      for(int row = 0; row < modelGroups[i]->rowCount(); row++)
        tbvTeam->setRowHeight(row, heightOfRows);
      tbvTeam->setMinimumHeight(heightOfRows * (modelGroups[i]->rowCount() + 1) + 5);

      tbvTeam->resize(columnsWidth + 5, heightOfRows * (modelGroups[i]->rowCount() + 1) + 5);

      connect(modelGroups[i], SIGNAL(itemChanged(QStandardItem*)),
              tbvTeam, SLOT(resizeRowsToContents()));
      connect(modelGroups[i], SIGNAL(itemChanged(QStandardItem*)),
              tbvTeam, SLOT(resizeColumnsToContents()));

      QVBoxLayout *vblDnDTeam = new QVBoxLayout;
      vblDnDTeam->addWidget(lblTeam);
      vblDnDTeam->addWidget(tbvTeam);
      vblDnDTeam->addSpacing(5);

      hblDnDTeams->addLayout(vblDnDTeam); 
      i++;
    }
    vblDnDTeams->addLayout(hblDnDTeams); 
    vblDnDTeams->addSpacing(10);
  }

  dlgDnDTeams->resize(350,420);

  if(dlgDnDTeams->exec() != QDialog::Accepted)
    return;

  dlgDnDTeams->close();
  delete dlgDnDTeams;

  createTeamsTab();
}

void ToMa::createTeamsTab()
{
  clearTeamsTab();

  for(int i = 0; i < modelGroups.count(); i++) {
    lblTeams << new QLabel(teamsNames[i]);
    modelTeams << new QStandardItemModel(0, 0, this);
    tbvTeams << new QTableView(this);
    tbvTeams[i]->setShowGrid(true);
    tbvTeams[i]->setModel(modelTeams[i]);
    tbvTeams[i]->setSelectionMode(QAbstractItemView::SingleSelection);
    tbvTeams[i]->setAlternatingRowColors(true);

    modelTeams[i]->setColumnCount(modelGroups[i]->rowCount() + columnsTeamsList.count());
    modelTeams[i]->setRowCount(modelGroups[i]->rowCount());
    for(int row = 0; row < modelGroups[i]->rowCount(); row++) {
      modelTeams[i]->setHeaderData(row, Qt::Horizontal, QString(charSmallFirst + row));
      modelTeams[i]->setHeaderData(row, Qt::Vertical, QString(charSmallFirst + row) + " " + modelGroups[i]->index(row, 0, QModelIndex()).data().toString());

      tbvTeams[i]->setItemDelegateForColumn(row, new ZTeamMatchesDelegate(this, i));
      for(int column = 0; column < modelGroups[i]->rowCount(); column++) {
        if(row != column) {
          QStandardItemModel *modelDelegate = new QStandardItemModel(2, 6);
          modelDelegate->setHeaderData(0, Qt::Vertical, QString(charSmallFirst + row)  + " " + modelGroups[i]->index(row, 0, QModelIndex()).data().toString());
          modelDelegate->setHeaderData(1, Qt::Vertical, QString(charSmallFirst + column)  + " " + modelGroups[i]->index(column, 0, QModelIndex()).data().toString());
          teamsPlayersModels[i][row][column] = modelDelegate;
        }
      }
    }

    for(int column = 0; column < columnsTeamsList.count(); column++)
      modelTeams[i]->setHeaderData(modelTeams[i]->columnCount() - columnsTeamsList.count() + column, Qt::Horizontal, columnsTeamsList.at(column));

    if(i)
      vblTeams->addSpacing(10);
    vblTeams->addWidget(lblTeams[i]);
    vblTeams->addWidget(tbvTeams[i]);

    tbvTeams[i]->resizeRowsToContents();
    tbvTeams[i]->resizeColumnsToContents();
 
    connect(modelTeams[i], SIGNAL(itemChanged(QStandardItem*)),
            tbvTeams[i], SLOT(resizeRowsToContents()));
    connect(modelTeams[i], SIGNAL(itemChanged(QStandardItem*)),
            tbvTeams[i], SLOT(resizeColumnsToContents()));

    int rowsHeight = tbvTeams[i]->rowHeight(0);
    if(tbvTeams[i]->horizontalScrollBar()->isVisible()) rowsHeight *= 2; else rowsHeight += 5;

    for(int row = 0; row < modelTeams[i]->rowCount(); row++)
      rowsHeight += tbvTeams[i]->rowHeight(row);
    tbvTeams[i]->setMinimumHeight(rowsHeight);

    for (int row = 0; row < modelTeams[i]->rowCount(); ++row)
      for (int column = 0; column < modelTeams[i]->columnCount(); ++column)
        modelTeams[i]->setData(modelTeams[i]->index(row, column, QModelIndex()), Qt::AlignCenter, Qt::TextAlignmentRole);
        //modelTeams[i]->setData(modelTeams[i]->index(row, column, QModelIndex()), Qt::AlignVCenter | Qt::AlignHCenter, Qt::TextAlignmentRole);

    for (int row = 0; row < modelTeams[i]->rowCount(); ++row) {
      modelTeams[i]->setData(modelTeams[i]->index(row, row, QModelIndex()), QBrush(Qt::darkGray), Qt::BackgroundRole);
      modelTeams[i]->item(row, row)->setFlags(0);
    }

    modelTeams[i]->setObjectName(QString("%1").arg(i));
    connect(modelTeams[i], SIGNAL(itemChanged(QStandardItem*)),
            this, SLOT(itemTeamsChange(QStandardItem*)));
  }

  tbwDock->setCurrentIndex(tabList.indexOf(tr("Groups")));
  tabPosReached = currentTab;
}

void ToMa::createSpidersTabData()
{
  clearSpidersTab();
  playersFinalOrder.clear();
  QMap<int, QString> playersMaps; 
  QMap<int, QString> playersMapss; 
  int pos = 0;
  int playersCount = 0;
  int playerssCount = 0;
  for(int i = 0; i < modelGroups.count(); ++i) {
    for(int x = 0; x < modelGroups[i]->rowCount(); x++) {
      if(i%2 == 0) 
        playersMaps[playersCount + modelTeams[i]->index(x, modelTeams[i]->columnCount() - 1, QModelIndex()).data().toUInt() - 1] = modelGroups[i]->index(x, 0, QModelIndex()).data().toString();
      else
        playersMapss[playersCount + modelTeams[i]->index(x, modelTeams[i]->columnCount() - 1, QModelIndex()).data().toUInt() - 1] = modelGroups[i]->index(x, 0, QModelIndex()).data().toString();
    }
    if(i % 2 == 1)
      playersCount += modelGroups[i]->rowCount();
  }

  pos = 0;
  playersCount = playersMaps.count();
  playerssCount = playersMapss.count();
  for(int i = 0; i < playersCount; i++) {
    if(playersCount > i + 1 && playerssCount > i + 1) {
      playersGroups[pos] << playersMaps[i] << playersMapss[i + 1] << playersMaps[i + 1] << playersMapss[i];
      i++;
    } else if(playersCount > i && playerssCount > i) {
      playersGroups[pos] << playersMaps[i] << playersMapss[i];
    } else if(playersCount > i && playerssCount <= i) {
      playersFinalOrder[playerssCount + i + 1] = playersMaps[i];
    }

    if(playersGroups[pos].count() == 4)
      pos++; 
  }

  for(QMap<int, QStringList>::iterator iterator = playersGroups.begin(); iterator != playersGroups.end(); ++iterator)
    if(playersGroups[iterator.key()].isEmpty())
      playersGroups.erase(iterator);
}

void ToMa::createSpidersTab()
{
  int place = 1;
  for(QMap<int, QStringList >::iterator iterator = playersGroups.begin(); iterator != playersGroups.end(); ++iterator ) { 
    wdtSpiders << new ZSpiderWidget(iterator.value());
    wdtSpiders[iterator.key()]->setPosition(place);
    vblSpiders->addWidget(wdtSpiders[iterator.key()]);
    vblSpiders->addSpacing(10);
    place += iterator.value().count();

    connect(wdtSpiders[iterator.key()], SIGNAL(matchDone(int, QMap<int, QString>)),
            this, SLOT(spiderMatchDone(int, QMap<int, QString>)));
    connect(wdtSpiders[iterator.key()], SIGNAL(windowModified(bool)),
            this, SLOT(setWindowModified(bool)));
  }

  tbwDock->setCurrentIndex(tabList.indexOf(tr("Spiders")));
  tabPosReached = currentTab;
}

void ToMa::moveUp()
{
  QModelIndexList selectedList = tbvList->selectionModel()->selectedRows();
  if(selectedList.count() == 1 && selectedList.at(0).row() > 0) {
    model->insertRow(selectedList.at(0).row() - 1, model->takeRow(selectedList.at(0).row()));
    tbvList->selectRow(selectedList.at(0).row() - 1);
    setWindowModified(true);
  }
}

void ToMa::moveDown()
{
  QModelIndexList selectedList = tbvList->selectionModel()->selectedRows();
  if(selectedList.count() == 1 && selectedList.at(0).row() < model->rowCount() - 1) {
    model->insertRow(selectedList.at(0).row() + 1, model->takeRow(selectedList.at(0).row()));
    tbvList->selectRow(selectedList.at(0).row() + 1);
    setWindowModified(true);
  }
}

void ToMa::moveTo()
{
  QModelIndexList selectedList = tbvList->selectionModel()->selectedRows();
  if(!selectedList.isEmpty() && selectedList.count() == 1) {
    QStringList items;
    for(int x = 0; x < model->rowCount(); x++)  
      items << QString("%1.").arg(x + 1);
    
    bool ok;
    QString item = QInputDialog::getItem(this, tr("%1 - goto").arg(prog),
                                               tr("Move to position:"), items, 0, false, &ok);
    if (ok && !item.isEmpty()) {
      for(int x = 0; x < model->columnCount(); x++) { 
        if(items.at(x) == item) {
          model->insertRow(x, model->takeRow(selectedList.at(0).row()));
          setWindowModified(true);
        }
      }
    }
  } else
    QMessageBox::information(this, prog, tr("Row must be selected for move action!"));
}

void ToMa::setCheckable(int row)
{
  QStandardItem *item = new QStandardItem();
  item->setCheckable(true);
  item->setCheckState(Qt::Unchecked);
  model->setItem(row, 0, item);
}

void ToMa::addLine()
{
  model->insertRow(model->rowCount());
  QStandardItem *item = new QStandardItem();
  item->setCheckable(true);
  model->setItem(model->rowCount() - 1, 0, item);
}

void ToMa::delLine()
{
  QModelIndexList selectedList;
  selectedList = tbvList->selectionModel()->selectedIndexes();

  if(!selectedList.isEmpty()) {
    QList<int> rows;
    for(int index = 0; index < selectedList.count(); index++)
      if(!rows.contains(selectedList.at(index).row()))
        rows << selectedList.at(index).row();
    qSort(rows.begin(), rows.end());
    model->removeRows((int) rows.first(), rows.count(), QModelIndex());
    setWindowModified(true);
  } else
    QMessageBox::information(this, prog, tr("Row must be selected for remove!"));
}

void ToMa::sort(quint8 column, Qt::SortOrder sortOrder)
{
  model->sort(column, sortOrder);
}

void ToMa::dlgSort(Qt::SortOrder sortOrder)
{
  QStringList items;
  for(int x = 0; x < model->columnCount(); x++)  
    items << columnsList.at(x);
  
  bool ok;
  QString item = QInputDialog::getItem(this, tr("%1 - Sorting").arg(prog),
                                             tr("Sort by:"), items, 0, false, &ok);
  if (ok && !item.isEmpty()) {
    for(int x = 0; x < model->columnCount(); x++) { 
      if(columnsList.at(x) == item)
        sort(x, sortOrder);
    }
  }

  setWindowModified(true);
}

void ToMa::sort()
{
  dlgSort(Qt::AscendingOrder);
}

void ToMa::sort_rev()
{
  dlgSort(Qt::DescendingOrder);
}

void ToMa::actionsEnable(bool active)
{
  addLineAct->setEnabled(active);
  delLineAct->setEnabled(active);
  sortAct->setEnabled(active);
  sortRevAct->setEnabled(active);
  moveUpAct->setEnabled(active);
  moveDownAct->setEnabled(active);
  moveToAct->setEnabled(active);
}

void ToMa::dockChanged(int index)
{
  currentTab = index;
  
  playAct->setEnabled(true);
  if(index == 0) {
    actionsEnable(true);
  } else if(index == 1) {
    actionsEnable(false);
  } else if(index == 3) {
    playAct->setEnabled(false);
  }
}

void ToMa::textResizeToContent()
{  
  if(!processingFlag) {
    tbvList->resizeColumnsToContents();
    for(int row = 0; row < model->rowCount(); row++)
      tbvList->setRowHeight(row, heightOfRows);
  }
}

void ToMa::playersUpdate()
{
  if(!processingFlag) {
    int presented = 0;
    for(int row = 0; row < model->rowCount(); row++) {
      if(model->item(row, 0)->checkState())
        presented++;
    }
    modelFinalOrder->setRowCount(presented);
    matchInfo->lblCount->setNum(model->rowCount());
    matchInfo->lblPresent->setNum(presented);
  }
}

void ToMa::leaguesUpdate()
{
  if(!processingFlag) {
    calcLeagues(); 
    for(QMap<int, int>::iterator iterator = leaguesNumbers.begin(); iterator != leaguesNumbers.end(); ++iterator)
      if(iterator.key() - 1 >= 0)
        matchInfo->lblLeagues[iterator.key() - 1]->setNum(iterator.value());
  }
}

void ToMa::calcLeagues()
{
  if(!processingFlag) {
    for(int index = 0; index < model->rowCount(); index++)
      if(!leagues.contains(model->data(model->index(index, leagueColumn, QModelIndex()), Qt::DisplayRole).toInt()))
        leagues << model->data(model->index(index, leagueColumn, QModelIndex()), Qt::DisplayRole).toInt();
    qSort(leagues.begin(), leagues.end());

    foreach(int league, leagues) {
      setLeagueProxy(league);
      leaguesNumbers[league] = 0;
      teamsMembers[league].clear();
      for(int row = 0; row < model->rowCount(); row++) {
        if(modelProxy->data(modelProxy->index(row, 0, QModelIndex()), Qt::CheckStateRole) == Qt::Checked) {
          leaguesNumbers[league]++;
          teamsMembers[league] << modelProxy->data(modelProxy->index(row, 1, QModelIndex()), Qt::DisplayRole).toString() + " " + modelProxy->data(modelProxy->index(row, 2, QModelIndex()), Qt::DisplayRole).toString();
        }
      }
    }
  }
}

void ToMa::setLeagueProxy(int league)
{
  modelProxy->setFilterKeyColumn(leagueColumn);
  modelProxy->setFilterRegExp(QString("%1").arg(league));
}

void ToMa::splitTeamMaxChange(int value)
{
  splitTeamMax = value;
}

void ToMa::itemChange(QStandardItem *item)
{
  if(!processingFlag) {
    setWindowModified(true);
    textResizeToContent();

    switch(item->column()){
      case 0: playersUpdate();
      case 5: leaguesUpdate(); break;
    }
  }
}

void ToMa::itemTeamsChange(QStandardItem *item)
{
  if(!processingFlag) {
    QStandardItemModel *mod = (QStandardItemModel*)sender();
    if(item->column() < mod->columnCount() - columnsTeamsList.count()) {
      processingFlag = true;
      int team = mod->objectName().toUInt();
      int rowAct = item->row();
      int columnAct = item->column();
      double balls = 0;
      double sets = 0;
      int ballsA = 0;
      int ballsB = 0;
      int points = 0;
      int setsA = 0;
      int setsB = 0;

      teamPlayersModel(team, columnAct, rowAct)->setColumnCount(teamPlayersModel(team, rowAct, columnAct)->columnCount());
      for(int column = 0; column < teamPlayersModel(team, rowAct, columnAct)->columnCount(); column++) {
        teamPlayersModel(team, columnAct, rowAct)->setData(teamPlayersModel(team, columnAct, rowAct)->index(0, column, QModelIndex()), teamPlayersModel(team, rowAct, columnAct)->index(1, column, QModelIndex()).data());
        teamPlayersModel(team, columnAct, rowAct)->setData(teamPlayersModel(team, columnAct, rowAct)->index(1, column, QModelIndex()), teamPlayersModel(team, rowAct, columnAct)->index(0, column, QModelIndex()).data());
      }

      for(int column = 0; column < mod->columnCount() - columnsTeamsList.count(); column++) {
        QString setMatch = mod->index(rowAct, column, QModelIndex()).data().toString();
        if(setMatch.contains(":")) {
          QStringList set = setMatch.split(":");
          setsA += set.at(0).toUInt();
          setsB += set.at(1).toUInt();
          if(set.at(0).toUInt() > set.at(1).toUInt())
            points++;
        }
        if(rowAct != column) {
          for(int col = 0; col < teamPlayersModel(team, rowAct, column)->columnCount() - 1; col++) {
            ballsA += teamPlayersModel(team, rowAct, column)->index(0, col, QModelIndex()).data().toUInt();
            ballsB += teamPlayersModel(team, rowAct, column)->index(1, col, QModelIndex()).data().toUInt();
          }
        }
      }
      balls = ballsB ? ballsA / (double) ballsB : (double) ballsA;
      sets = setsB ? setsA / (double) setsB : (double) setsA;

      bool allMatchesCompleted = true;
      for(int row = 0; row < mod->rowCount(); row++) {
        for(int column = 0; column < mod->columnCount() - columnsTeamsList.count(); column++) {
          if(row != column) {
            if(mod->index(row, column, QModelIndex()).data().toString() == "")
              allMatchesCompleted = false;
          }
        }
      }

      modelTeams[team]->setData(modelTeams[team]->index(rowAct, modelTeams[team]->columnCount() - columnsTeamsList.count() + columnsTeamsList.indexOf(tr("Balls")), QModelIndex()), QString::number(balls, 'f', 2), Qt::EditRole);
      modelTeams[team]->setData(modelTeams[team]->index(rowAct, modelTeams[team]->columnCount() - columnsTeamsList.count() + columnsTeamsList.indexOf(tr("Sets")), QModelIndex()), QString::number(sets, 'f', 2), Qt::EditRole);
      modelTeams[team]->setData(modelTeams[team]->index(rowAct, modelTeams[team]->columnCount() - columnsTeamsList.count() + columnsTeamsList.indexOf(tr("Points")), QModelIndex()), points, Qt::EditRole);
      if(allMatchesCompleted) {
        QStandardItemModel *modelPro = new QStandardItemModel(0, 0, this);
        
        modelPro->setRowCount(mod->rowCount());
        modelPro->setColumnCount(columnsTeamsList.count() + 1);
        for(int row = 0; row < modelPro->rowCount(); row++) {
          modelPro->setData(modelPro->index(row, 0, QModelIndex()), modelGroups[team]->index(row, 0, QModelIndex()).data().toString());
          modelPro->setData(modelPro->index(row, 1, QModelIndex()), modelTeams[team]->index(row, modelTeams[team]->columnCount() - columnsTeamsList.count() + 0, QModelIndex()).data().toDouble());
          modelPro->setData(modelPro->index(row, 2, QModelIndex()), modelTeams[team]->index(row, modelTeams[team]->columnCount() - columnsTeamsList.count() + 1, QModelIndex()).data().toDouble());
          modelPro->setData(modelPro->index(row, 3, QModelIndex()), modelTeams[team]->index(row, modelTeams[team]->columnCount() - columnsTeamsList.count() + 2, QModelIndex()).data().toDouble());
        }

        modelPro->sort(1, Qt::DescendingOrder);
        modelPro->sort(2, Qt::DescendingOrder);
        modelPro->sort(3, Qt::DescendingOrder);

        QStringList teamGroup;
        for(int row = 0; row < modelGroups[team]->rowCount(); row++)
          teamGroup << modelGroups[team]->index(row, 0, QModelIndex()).data().toString();
        int order = 1;
        for(int row = 0; row < modelPro->rowCount(); row++) {
          modelPro->setData(modelPro->index(row, 4, QModelIndex()), order, Qt::EditRole);
          modelTeams[team]->setData(modelTeams[team]->index(teamGroup.indexOf(modelPro->index(row, 0, QModelIndex()).data().toString()), modelTeams[team]->columnCount() - columnsTeamsList.count() + columnsTeamsList.indexOf(tr("Order")), QModelIndex()), order);
          order++;
        }
      }
      
      tbvTeams[team]->resizeRowsToContents();
      tbvTeams[team]->resizeColumnsToContents();
      
      if(!curFile.isEmpty()) {
        QFileInfo curFileInfo = QFileInfo(curFile);
        if(curFileInfo.suffix() != fileTournamentSuffix) {
          curFile.replace(curFileInfo.suffix(), fileTournamentSuffix);
          setCurrentFile(curFile);
        }
      }
      fileTournamentSuffixEnabled = true; 
    }
    setWindowModified(true);
    processingFlag = false;
  }
}

void ToMa::spiderMatchDone(int pos, QMap<int, QString> winsOrder)
{
  bool filledAll = true;
  for(QMap<int, QString>::iterator iterator = winsOrder.begin(); iterator != winsOrder.end(); ++iterator)
    if(iterator.value() == "")
      filledAll = false;

  if(filledAll) {
    for(QMap<int, QString>::iterator iterator = winsOrder.begin(); iterator != winsOrder.end(); ++iterator) {
      playersFinalOrder[pos + iterator.key()] = iterator.value();
    }
    
    spiderFinalOrderMaker();
  }
}

bool ToMa::spiderFinalOrderMaker()
{
  bool matchesDone = true;
  for(int x = 0; x < wdtSpiders.count(); x++)
    if(!wdtSpiders[x]->matchesDone())
      matchesDone = false;;

  if(matchesDone) {
    for(int row = 0; row < model->rowCount(); row++) {
      QString name = model->data(model->index(row, 1, QModelIndex()), Qt::DisplayRole).toString() + " "
                   + model->data(model->index(row, 2, QModelIndex()), Qt::DisplayRole).toString();
      for(QMap<int, QString>::iterator iterator = playersFinalOrder.begin(); iterator != playersFinalOrder.end(); ++iterator) {
        if(playersFinalOrder[iterator.key()] == name) {
          for(int column = 1; column < columnsList.count() - 1; column++) {
            modelFinalOrder->setData(modelFinalOrder->index(iterator.key() - 1, column - 1, QModelIndex()), model->data(model->index(row, column, QModelIndex()), Qt::DisplayRole).toString());
          }
        }
      }
    }
    tbvFinalOrder->resizeRowsToContents();
    tbvFinalOrder->resizeColumnsToContents();
    tbwDock->setCurrentIndex(tabList.indexOf(tr("Final Order")));
    tabPosReached = currentTab;
  }

  return matchesDone;
}

void ToMa::trayIconActivated(QSystemTrayIcon::ActivationReason reason)
{
  switch (reason) {
    case QSystemTrayIcon::Trigger: 
        showNormal();
        show();
        activateWindow();
      break;
    case QSystemTrayIcon::DoubleClick: break;
    default: break; 
  }
}

void ToMa::keyPressEvent(QKeyEvent *event)
{
  if(event->key() == Qt::Key_Return) {
    QModelIndexList selectedList = tbvList->selectionModel()->selectedIndexes();
    if(selectedList.count()) {
      tbvList->selectRow(selectedList.at(selectedList.count() - 1).row() + 1);
      if((selectedList.at(0).row() == model->rowCount() - 1) && (model->item(model->rowCount() - 1, 0))) {
        if(model->item(model->rowCount() - 1, 0)->text() != "") {
          model->setRowCount(model->rowCount()+1);
          tbvList->selectRow(model->rowCount()-1);
          setCheckable(model->rowCount()-1);
        }
      }
    }
  }
}

// Date("dd.MM.yyyy");
QString ToMa::date(QString format)
{
  QDate date = QDate::currentDate();
  return date.toString(format);
}

// time("hh:mm:ss");
QString ToMa::time(QString format)
{
  QTime time = QTime::currentTime(); 
  return time.toString(format);
}

// returns app name
QString ToMa::appName()
{
  return prog;
}

QStandardItemModel *ToMa::teamPlayersModel(int team, int row, int column)
{
  return teamsPlayersModels[team][row][column];
}

void ToMa::setTabText()
{
  for(int tab = 0; tab < tabList.count(); tab++)
    tbwDock->setTabText(tab, tabList.at(tab));
}

void ToMa::setHeaderData()
{
  model->setHeaderData(0, Qt::Horizontal, tr(""));

  for(int column = 0; column < columnsList.count(); column++)
    model->setHeaderData(column, Qt::Horizontal, columnsList.at(column));

  for(int i = 0; i < modelGroups.count(); i++)
    for(int column = 0; column < columnsTeamsList.count(); column++)
      modelTeams[i]->setHeaderData(modelTeams[i]->columnCount() - columnsTeamsList.count() + column, Qt::Horizontal, columnsTeamsList.at(column));

  for(int column = 0; column < columnsFinalOrderList.count(); column++)
    modelFinalOrder->setHeaderData(column, Qt::Horizontal, columnsFinalOrderList.at(column));
}

//reload names
void ToMa::namesReload()
{
  newAct->setText(tr("&New"));
  newAct->setShortcut(tr("Ctrl+N"));
  newAct->setStatusTip(tr("Create a new file"));

  openAct->setText(tr("&Open..."));
  openAct->setShortcut(tr("Ctrl+O"));
  openAct->setStatusTip(tr("Open an existing file"));
  
  saveAct->setText(tr("&Save"));
  saveAct->setShortcut(tr("Ctrl+S"));
  saveAct->setStatusTip(tr("Save the document to disk"));

  saveAsAct->setText(tr("Save &As"));
  saveAsAct->setStatusTip(tr("Save the document under a new name"));

  exitAct->setText(tr("E&xit"));
  exitAct->setShortcut(tr("Ctrl+Q"));
  exitAct->setStatusTip(tr("Exit the application"));

  addLineAct->setText(tr("A&dd Line"));
  addLineAct->setShortcut(tr("Ctrl+Ins"));
  addLineAct->setStatusTip(tr("Add new line"));

  delLineAct->setText(tr("D&elete Line"));
  delLineAct->setShortcut(tr("Ctrl+Del"));
  delLineAct->setStatusTip(tr("Delete selected line"));
  
  sortAct->setText(tr("So&rt"));
  sortAct->setShortcut(tr("Ctrl+R"));
  sortAct->setStatusTip(tr("Sort lines"));

  sortRevAct->setText(tr("Sort Rever&t"));
  sortRevAct->setShortcut(tr("Ctrl+T"));
  sortRevAct->setStatusTip(tr("Sort lines revert"));

  playAct->setText(tr("&Play"));
  playAct->setShortcut(tr("Ctrl+P"));
  playAct->setStatusTip(tr("Play tournament"));

  moveUpAct->setText(tr("Move &up"));
  moveUpAct->setShortcut(tr("Ctrl+U"));
  moveUpAct->setStatusTip(tr("Move row up"));

  moveDownAct->setText(tr("Move &down"));
  moveDownAct->setShortcut(tr("Ctrl+D"));
  moveDownAct->setStatusTip(tr("Move row down"));

  moveToAct->setText(tr("Move &to"));
  moveToAct->setShortcut(tr("Ctrl+T"));
  moveToAct->setStatusTip(tr("Move row to"));


  updateAct->setText(tr("&Update application"));
  updateAct->setStatusTip(tr("Check new version and update application"));

  aboutAct->setText(tr("&About"));
  aboutAct->setStatusTip(tr("Show the application's About box"));

  fileMenu->setTitle(tr("&File"));
  editMenu->setTitle(tr("&Edit"));
  toolsMenu->setTitle(tr("&Tools"));
  optionsMenu->setTitle(tr("&Options"));

  charCodeSubMenu->setTitle(tr("Character &Encoding"));
  styleSubMenu->setTitle(tr("&Style"));
  langSubMenu->setTitle(tr("&Language"));
  helpMenu->setTitle(tr("&Help"));
 
  minAct->setText(tr("Mi&nimize"));
  maxAct->setText(tr("Ma&ximize"));
  restAct->setText(tr("&Restore"));

  matchInfo->lblCountText->setText(tr("Count: "));
  matchInfo->lblPresentText->setText(tr("Present: "));
  matchInfo->lblGroupMaxText->setText(tr("Group max: "));
  matchInfo->lblLeagueText->setText(tr("--- Leagues ---"));
  for(int i = 0; i < leaguesMax; i++)
    matchInfo->lblLeaguesText[i]->setText(tr("%1. league: ").arg(i + 1));

  setApplicationData();
  reloadApplicationData();
  setTabText();
  setHeaderData(); 
  textResizeToContent();


  bool modified = isWindowModified();
  setCurrentFile(curFile);
  setWindowModified(modified);
}

void ToMa::changeModelLanguage(QString lang)
{
  changeLanguage(lang);
  setApplicationData();
  reloadApplicationData();
  setHeaderData();
}

//change language from string
void ToMa::changeLanguage(QString lang)
{
  translator.load(":/toma_" + lang + ".gm");
}

//change language from action
void ToMa::changeLanguage(QAction* action)
{
  changeLanguage(langShortMap.key(action->data().toString()));
  namesReload();
}

//change character code 
void ToMa::changeCharCode(QAction* action)
{
  fileEncoding = action->data().toString();
}

//change style from string
void ToMa::changeStyle(QString style)
{
  QApplication::setStyle(QStyleFactory::create(style));
}

//change style from action
void ToMa::changeStyle(QAction* action)
{
  changeStyle(action->data().toString());
}

void ToMa::readSettings()
{
  QSettings settings(QApplication::applicationDirPath() + "/" + _appName + ".ini", QSettings::IniFormat);
  if(settings.contains("RecentFiles")) {
    recentFiles = settings.value("RecentFiles").toStringList();
    updateRecentFileActions();
  }

  fileEncoding = settings.value("FileEncoding", "UTF-8").toString();
  charCodeMap[fileEncoding]->setChecked(true);
  
  preferredLanguage = settings.value("Language", preferredLanguage).toString();
  languageMap[langShortMap[preferredLanguage]]->setChecked(true);
  languageMap[langShortMap[preferredLanguage]]->trigger();

  styleMap[settings.value("Style", QApplication::style()->objectName()).toString()]->setChecked(true);
  changeStyle(settings.value("Style", QApplication::style()->objectName()).toString());
}

//write settings
void ToMa::writeSettings()
{
  QSettings settings(QApplication::applicationDirPath() + "/" + _appName + ".ini", QSettings::IniFormat);

  if(recentFiles.isEmpty()) {
    settings.setValue("RecentFiles", "");
  } else
    settings.setValue("RecentFiles", recentFiles);

  foreach(QString lang, langShortMap) {
    if(languageMap[lang]->isChecked()) {
      settings.setValue("Language", langShortMap.key(lang));
    }
  }

  bool styleFound = false;
  foreach(QString style, QStyleFactory::keys()) {
    if(styleMap[style]->isChecked()) {
      settings.setValue("Style", style);
      styleFound = true;
    }
  }
  if(!styleFound)
    settings.setValue("Style", QApplication::style()->objectName());
  
  bool fileEncodingFound = false;

  foreach(QString charCode, charCodeList) {
    if(charCodeMap[charCode]->isChecked()) {
      settings.setValue("FileEncoding", charCode);
      fileEncodingFound = true;
    }  
  }
  if(!fileEncodingFound) 
    settings.setValue("FileEncoding", "UTF-8");
}

//read tournament settings
void ToMa::readTournamentSettings(const QString &fileName)
{
  processingFlag = true;
  QSettings settings(fileName, QSettings::IniFormat);
  
  changeModelLanguage("en");
  
  leagues.clear();
  QList<QVariant> leaguesVar = settings.value("leagues").toList();
  foreach(QVariant var, leaguesVar)
    leagues << var.toInt();
  splitTeamMax = settings.value("splitTeamMax", splitTeamMax).toInt();
  tabPosReached = settings.value("tabPosReached", tabPosReached).toInt();

  int size = settings.beginReadArray("attendanceList");
  model->setRowCount(0);
  model->setRowCount(size);
  for(int row = 0; row < size; row++) {
    settings.setArrayIndex(row);
    setCheckable(row);
    model->item(row, 0)->setCheckState(static_cast<Qt::CheckState> (settings.value("present").toUInt()));
    for(int column = 0; column < columnsList.count(); column++) {
      model->setData(model->index(row, column, QModelIndex()), settings.value(columnsList[column]).toString());
    }
  }
  settings.endArray();
  
  leaguesNumbers.clear();
  size = settings.beginReadArray("leaguesNumbers");
  for(int cnt = 1; cnt < size; cnt++) {
    settings.setArrayIndex(cnt);
    leaguesNumbers[cnt] = settings.value("num").toInt();
  }
  settings.endArray();
  
  leagues.clear();
  leaguesNumbers.clear();
  teamsGroups.clear();
  size = settings.beginReadArray("teamsGroups");
  for(int cnt = 1; cnt < size; cnt++) {
    settings.setArrayIndex(cnt);
    int sizee = settings.beginReadArray("teams");
    for(int i = 0; i < sizee; ++i) {
      settings.setArrayIndex(i);
      teamsGroups[cnt] << settings.value("cnt").toInt();
    }
    settings.endArray();
  }
  settings.endArray();

  teamsMembers.clear();
  size = settings.beginReadArray("teamsMembers");
  for(int cnt = 1; cnt < size; cnt++) {
    settings.setArrayIndex(cnt);
    teamsMembers[cnt] = settings.value("members").toStringList();
  }
  settings.endArray();
  
  teamsNames.clear();
  teamsNames = settings.value("teamsNames", "").toStringList();

  size = settings.beginReadArray("teamsModelsGroups");
  for(int i = 0; i < size; i++) {
    settings.setArrayIndex(i);
    QStringList singleGroup = settings.value("groups").toStringList();
    modelGroups << new QStandardItemModel(singleGroup.count(), 1, this);
    for(int row = 0; row < modelGroups[i]->rowCount(); row++)
      modelGroups[i]->setData(modelGroups[i]->index(row, 0, QModelIndex()), singleGroup[row]);
  }
  settings.endArray();

  teamsPlayersModels.clear();
  createTeamsTab();

  size = settings.beginReadArray("teamsPlayersModels");
  for(int cnt = 0; cnt < size; cnt++) {
    settings.setArrayIndex(cnt);
    int sizeT = settings.beginReadArray("teams");
    for(int cntT = 0; cntT < sizeT; cntT++) {
      for(int i = columnsTeamsList.count(); i > 0; i--) {
        int columnResult = modelTeams[cnt]->columnCount() - i;
        modelTeams[cnt]->setData(modelTeams[cnt]->index(cntT, columnResult, QModelIndex()), settings.value(columnsTeamsList.at(4 - i), ""));
      }
      settings.setArrayIndex(cntT);
      int sizeP = settings.beginReadArray("players");
      for(int cntP = 0; cntP < sizeP; cntP++) {
        settings.setArrayIndex(cntP);
        modelTeams[cnt]->setData(modelTeams[cnt]->index(cntT, cntP, QModelIndex()), settings.value("score"));
        int sizeM = settings.beginReadArray("match");
        for(int row = 0; row < sizeM; row++) {
          settings.setArrayIndex(row);
          teamsPlayersModels[cnt][cntT][cntP]->setHeaderData(row, Qt::Vertical, settings.value("line"));
          int sizeR = settings.beginReadArray("records");
          teamsPlayersModels[cnt][cntT][cntP]->setColumnCount(sizeR);
          for(int column = 0; column < sizeR; column++) {
            settings.setArrayIndex(column);
            teamsPlayersModels[cnt][cntT][cntP]->setData(teamsPlayersModels[cnt][cntT][cntP]->index(row, column, QModelIndex()), settings.value("points"));
          }
          settings.endArray();
        }
        settings.endArray();
      }
      settings.endArray();
    }
    settings.endArray();
  }
  settings.endArray(); 
 
  createSpidersTabData();
  size = settings.beginReadArray("playersGroups");
  for(int cnt = 0; cnt < size; cnt++) {
    settings.setArrayIndex(cnt);
    playersGroups[cnt] = settings.value("players").toStringList();
    playersGroupsResults[cnt] = settings.value("playersResults").toStringList();
  }
  settings.endArray();
  
  if(size) {
    createSpidersTab();
    for(QMap<int, QStringList>::iterator iterator = playersGroupsResults.begin(); iterator != playersGroupsResults.end(); ++iterator) {
      wdtSpiders[iterator.key()]->setResults(iterator.value());
      wdtSpiders[iterator.key()]->spiderUpdateAll();
    }
  }

  size = settings.beginReadArray("finalOrderModel");
  modelFinalOrder->setRowCount(0);
  modelFinalOrder->setRowCount(size);
  for(int row = 0; row < size; row++) {
    settings.setArrayIndex(row);
    for(int column = 0; column < columnsFinalOrderList.count(); column++) {
      modelFinalOrder->setData(modelFinalOrder->index(row, column, QModelIndex()), settings.value(columnsFinalOrderList[column]).toString());
    }
  }
  settings.endArray();

  fileTournamentSuffixEnabled = true;
  changeModelLanguage(preferredLanguage);
  processingFlag = false;
  textResizeToContent();
}

//write tournament settings
void ToMa::writeTournamentSettings(const QString &fileName)
{
  QSettings settings(fileName, QSettings::IniFormat);
  
  changeModelLanguage("en");
  
  QList<QVariant> leaguesVar;
  foreach(int var, leagues)
    leaguesVar << QVariant(var);
  settings.setValue("leagues", leaguesVar);
  settings.setValue("splitTeamMax", splitTeamMax);
  settings.setValue("tabPosReached", tabPosReached);

  settings.beginWriteArray("attendanceList");
  for(int row = 0; row < model->rowCount(); row++) {
    settings.setArrayIndex(row);
    settings.setValue("present", model->item(row, 0)->checkState());
    for(int column = 0; column < columnsList.count(); column++) {
      settings.setValue(columnsList[column], model->index(row, column, QModelIndex()).data().toString());
    }
  }
  settings.endArray();

  settings.beginWriteArray("leaguesNumbers");
  for(QMap<int, int>::iterator iterator = leaguesNumbers.begin(); iterator != leaguesNumbers.end(); ++iterator ) {
    settings.setArrayIndex(iterator.key());
    settings.setValue("num", iterator.value());
  }
  settings.endArray();

  settings.beginWriteArray("teamsGroups");
  for(QMap<int, QVector<int> >::iterator iterator = teamsGroups.begin(); iterator != teamsGroups.end(); ++iterator ) {
    settings.setArrayIndex(iterator.key());
    settings.beginWriteArray("teams");
    for (int i = 0; i < iterator.value().size(); ++i) {
      settings.setArrayIndex(i);
      settings.setValue("cnt", iterator.value().at(i));
    }
    settings.endArray();
  }
  settings.endArray();

  settings.beginWriteArray("teamsMembers");
  for(QMap<int, QStringList>::iterator iterator = teamsMembers.begin(); iterator != teamsMembers.end(); ++iterator ) {
    settings.setArrayIndex(iterator.key());
    settings.setValue("members", iterator.value());
  }
  settings.endArray();

  settings.setValue("teamsNames", teamsNames);

  settings.beginWriteArray("teamsModelsGroups");
  settings.setValue("size", modelGroups.count());
  for(int i = 0; i < modelGroups.count(); i++) {
    settings.setArrayIndex(i);
    QStringList singleGroup;
    for(int row = 0; row < modelGroups[i]->rowCount(); row++)
      singleGroup << modelGroups[i]->index(row, 0, QModelIndex()).data().toString();
    settings.setValue("groups", singleGroup);
  }
  settings.endArray();

  settings.beginWriteArray("teamsPlayersModels");
  for(QMap<int, QMap<int, QMap<int, QStandardItemModel*> > >::iterator iterator1 = teamsPlayersModels.begin(); iterator1 != teamsPlayersModels.end(); ++iterator1 ) {
    settings.setArrayIndex(iterator1.key());
    settings.beginWriteArray("teams");
    for(QMap<int, QMap<int, QStandardItemModel*> >::iterator iterator2 = iterator1.value().begin(); iterator2 != iterator1.value().end(); ++iterator2 ) {
      for(int i = columnsTeamsList.count(); i > 0; i--) {
        int columnResult = modelTeams[iterator1.key()]->columnCount() - i;
        settings.setValue(columnsTeamsList.at(4 - i), modelTeams[iterator1.key()]->index(iterator2.key(), columnResult, QModelIndex()).data().toString());
      }
      settings.setArrayIndex(iterator2.key());
      settings.beginWriteArray("players");
      for(QMap<int, QStandardItemModel*>::iterator iterator3 = iterator2.value().begin(); iterator3 != iterator2.value().end(); ++iterator3 ) {
        settings.setArrayIndex(iterator3.key());
        QStandardItemModel *modit = iterator3.value();
        settings.setValue("score", modelTeams[iterator1.key()]->index(iterator2.key(), iterator3.key(), QModelIndex()).data().toString());
        settings.beginWriteArray("match");
        for(int row = 0; row < modit->rowCount(); row++) {
          settings.setArrayIndex(row);
          settings.setValue("line", modit->headerData(row, Qt::Vertical));
          settings.beginWriteArray("records");
          for(int column = 0; column < modit->columnCount(); column++) {
            settings.setArrayIndex(column);
            settings.setValue("points", modit->index(row, column, QModelIndex()).data().toString());
          }
          settings.endArray();
        }
        settings.endArray();
      }
      settings.endArray();
    }
    settings.endArray();
  }
  settings.endArray();

  settings.beginWriteArray("playersGroups");
  if(wdtSpiders.size()) {
    for(QMap<int, QStringList>::iterator iterator = playersGroups.begin(); iterator != playersGroups.end(); ++iterator ) {
      settings.setArrayIndex(iterator.key());
      settings.setValue("players", iterator.value());
      settings.setValue("playersResults", wdtSpiders[iterator.key()]->results());
    }
  }
  settings.endArray();

  settings.beginWriteArray("finalOrderModel");
  settings.setValue("size",modelFinalOrder->rowCount());
  for(int row = 0; row < modelFinalOrder->rowCount(); row++) {
    settings.setArrayIndex(row);
    for(int column = 0; column < columnsFinalOrderList.count(); column++) {
      settings.setValue(columnsFinalOrderList[column], modelFinalOrder->index(row, column, QModelIndex()).data().toString());
    }
  }
  settings.endArray();

  changeModelLanguage(preferredLanguage);
}

void ToMa::zdownloaderGotVersion(QString ver)
{
  if(progVer != ver) {
    gotNewVersion = true;
    QMessageBox::information(this, prog, tr("New version is available!\n Update is recommended."));
  }
}

