#ifndef TOMA_H
#define TOMA_H

#include <QtGui>
#include <QSystemTrayIcon>
#include <QMainWindow>
#include <QThread>
#include <QTranslator>
#include <QMap>

#include "zspiderwidget.h"
#include "zteammatchesdelegate.h"
#include "zdownloader.h"
#include "zbasicexcel.h"

using namespace ZExcel;

class QScrollArea;
class QObject;
class QMenu;
class QFrame;
class QTabWidget;
class QTableView;
class QAction;
class QActionGroup;
class QStandardItem;
class QStandardItemModel;
class QProxyModel;
class QSortFilterProxyModel;
class QModelIndex;
class QString;
class QLabel;
class QImage;
class QDate;
class QTime;
class QTextStream;

class ToMa : public QMainWindow
{
    Q_OBJECT

public:
    ToMa();

    QString time(QString format);
    QString date(QString format);
    QString appName();
    QStandardItemModel *teamPlayersModel(int team, int row, int column);

signals:
    
protected:
    void keyPressEvent(QKeyEvent *event);
    void closeEvent(QCloseEvent *event);

private slots:
    void newFile();
    void open();
    bool save();
    bool saveAs();
    void openRecentFile();
    void clear();
    void addLine();
    void delLine();
    void play();
    void moveUp();
    void moveDown();
    void moveTo();
    void update();
    void about();
    void changeModelLanguage(QString lang);
    void changeLanguage(QString lang);
    void changeLanguage(QAction* action);
    void changeStyle(QString style);
    void changeStyle(QAction* action);
    void changeCharCode(QAction* action);
    void itemChange(QStandardItem *item); 
    void itemTeamsChange(QStandardItem *item); 
    void splitTeamMaxChange(int value);
    void spiderMatchDone(int pos, QMap<int, QString> winsOrder);

    void sort();
    void sort_rev();
    void statusBarMessageChanged(const QString &message);
    void dockChanged(int index);
    void trayIconActivated(QSystemTrayIcon::ActivationReason reason);
    
    void zdownloaderGotVersion(QString ver);

private:
    void createActions();
    void createMenus();
    void createContextMenu();
    void createToolBars();
    void createStatusBar();
    void createTrayIcon();
    void clearTeamsTab();
    void createTeamsTab();
    void clearSpidersTab();
    void createSpidersTab();
    void createSpidersTabData();
    void readSettings();
    void writeSettings();
    bool okToContinue();
    void setCurrentFile(const QString &fileName);
    void updateRecentFileActions();
    QString strippedName(const QString &fullFileName);
    bool loadFile(const QString &fileName);
    bool readFile(const QString &fileName);
    bool readXLSFileToModel(const QString &fileName);
    void readToModel(QTextStream &stream, QString &streamEncoding);
    bool saveFile(const QString &fileName);
    bool writeFile(const QString &fileName);
    bool writeXLSFile(const QString &fileName);

    void readTournamentSettings(const QString &fileName);
    void writeTournamentSettings(const QString &fileName);

    void statusLoadFailedMessageRestoreCursor();
    void statusSaveFailedMessageRestoreCursor();
    
    void setCheckable(int row);
    void namesReload();
    void setApplicationData();
    void reloadApplicationData();
    void setTabText();
    void setHeaderData();
    void textResizeToContent();

    void actionsEnable(bool active);
    void sort(quint8 column, Qt::SortOrder sortOrder);
    void dlgSort(Qt::SortOrder sortOrder);

    void playersUpdate();
    void leaguesUpdate();
    void calcLeagues();
    void setLeagueProxy(int league);
    bool spiderFinalOrderMaker();
    
    QString prg;
    QString prog;
    QString progVer;
    QString _appName;

    int tabPosReached;

    char charSmallFirst;
    char charBigFirst;

    QString fileTournamentSuffix;
    QString fileTournamentSuffixLabel;

    QString curFile;
    QString fileEncoding;
    QString preferredLanguage;

    int leaguesMax;
    int leagueColumn;
    int splitTeamMax;

    QStringList columnsList;
    QStringList columnsTeamsList;
    QStringList columnsFinalOrderList;
    QStringList tabList;
    QStringList recentFiles;

    QStandardItemModel *model;
    QSortFilterProxyModel *modelProxy;
    QStandardItemModel *modelFinalOrder;

    QTabWidget *tbwDock; 
    QTableView *tbvList;
    QTableView *tbvFinalOrder;
    QFrame *frmTeams;
    QScrollArea *slaTeams;
    QFrame *frmSpiders;
    QScrollArea *slaSpiders;

    QTranslator translator;

    QList<QByteArray> charCodeList;
    QList<QString> languageList;
    QMap<QString, QString> langShortMap; 
    QMap<QString, QAction*> languageMap; 
    QMap<QString, QAction*> styleMap; 
    QMap<QString, QAction*> charCodeMap; 
    
    QList<int> leagues;
    QMap<int, int> leaguesNumbers;
    QStringList teamsNames;
    QMap<int, QVector<int> > teamsGroups;
    QMap<int, QStringList> teamsMembers;
    QVector<QStandardItemModel*> modelGroups;
    QMap<int, QMap<int, QMap<int, QStandardItemModel*> > > teamsPlayersModels;

    QVector<QLabel*> lblTeams;
    QVector<QTableView*> tbvTeams;
    QVector<QStandardItemModel*> modelTeams;
    QVBoxLayout *vblTeams;

    QMap<int, QString> playersFinalOrder;
    QMap<int, QStringList> playersGroups;
    QMap<int, QStringList> playersGroupsResults;

    QVector<ZSpiderWidget*> wdtSpiders;
    QVBoxLayout *vblSpiders;

    QMenu *trayIconMenu;
    QIcon icon;
    QSystemTrayIcon *trayIcon;

    QMenu *fileMenu;
    QMenu *editMenu;
    QMenu *toolsMenu;
    QMenu *optionsMenu;
    QMenu *charCodeSubMenu;
    QMenu *styleSubMenu;
    QMenu *langSubMenu;
    QMenu *helpMenu;

    QToolBar *fileToolBar;
    QToolBar *editToolBar;

    QActionGroup *charCodeActGroup; 
    QActionGroup *styleActGroup; 
    QActionGroup *languageActGroup; 

    QAction *newAct;
    QAction *openAct;
    QAction *saveAct;
    QAction *saveAsAct;
    QAction *importMatchAct;
    QAction *exportMatchAct;
    QAction *printAct;
    QAction *exitAct;

    QAction *addLineAct;
    QAction *delLineAct;
    QAction *sortAct;
    QAction *sortRevAct;
    QAction *goToCellAct;
    QAction *separatorAct;
    QAction *playAct;
    QAction *moveUpAct;
    QAction *moveDownAct;
    QAction *moveToAct;
    QAction *updateAct;
    QAction *aboutAct;

    QAction *minAct;
    QAction *maxAct;
    QAction *restAct;

    enum { MaxRecentFiles = 5 };
    QAction *recentFileActs[MaxRecentFiles];

    struct ZMatchInfo {
      QLabel *lblCountText;
      QLabel *lblCount;
      QLabel *lblPresentText;
      QLabel *lblPresent;
      QLabel *lblGroupMaxText;
      QLabel *lblLeagueText;
      QVector<QLabel *> lblLeaguesText;
      QVector<QLabel *> lblLeagues;
      QSpinBox *spbTeamMax;
    } *matchInfo;

    ZDownloader *zdownloader;
    bool gotNewVersion;
};
#endif
