#ifndef ZDOWNLOADER_H
#define ZDOWNLOADER_H
 
#include <QtGui/QMainWindow>
#include <QFile>
#include <QtNetwork>
#include <QMessageBox>
#include <QTranslator>
 
class QFrame;
class QLabel;
class QString;
class QByteArray;
class QTimer;
class QDateTime;
class QPushButton;
class QProgressBar;
class QHBoxLayout;
class QVBoxLayout;

class ZDownloader : public QWidget
{
	Q_OBJECT
 
public:
  enum Target {Default = 0, ToMa = 1};
  Q_DECLARE_FLAGS(Targets, Target);

	ZDownloader(QWidget *parent = 0, Qt::WindowFlags flags = 0, bool standalone = true);
	~ZDownloader();
  void downloadOnce(QString source);
  QString appName();
  QString appVersion();
  QString version(bool baseApp);
  QString osVer();
  QString osSuffix();
  QString self();
  QString dataSumarize(double size, bool speed);
	void downloadVersions();
  void setTarget(int target);

public slots:
	void downloadPressed();
	void downloadProgress(qint64 bytesReceived, qint64 bytesTotal);
	void downloadFinished();
	void downloadReadyRead();
	void cancelDownload();
  void showDownloadInfo();

signals:
  void gotVersion(QString ver);

private:
	void download(QString source);
	QFile *file;
	QNetworkAccessManager manager;
	QNetworkReply *reply;
	bool downloadRequestAborted;

  QString preferredLanguage;
  QTranslator translator;

  QLabel *lblInfo;
  QLabel *lblDownloadedName;
  QLabel *lblSpeed;
  QLabel *lblData;
  QProgressBar *pbrDownload;
  QPushButton *btnDownload;
  QPushButton *btnCancelDownload;
  QPushButton *btnExit;
  QProcess *p;
  QTimer *tmrDownloadInfo;

  int _target;
  QString targetAppName;
  QString targetAppNamePath;
  QString targetAppInfo;
  QString targetAppTitle;

  QString _appName;
  QString _appVersion;
  QString _appLocation;
  QString downloadPath;
  QString downloadVersion;
  QString downloadFile;
  QString downloadCurrent;
  QString _currentOS;
  QString _currentOSsuffix;
  QByteArray versions;
  bool downloadingVersionFlag;

  QDateTime m_dtStartTime;
  qint64 avgDownloadSpeed;    
  qint64 downloadBytesReceived;    
  qint64 downloadBytesTotal;    
};
 
#endif

