#include "zspiderwidget.h"
#include <QPainter>

int rowHeight = 32;
int rowWidthOffset = 10;
int rowTextOffset = 5;
int rowOffset = 5;


ZSpiderWidget::ZSpiderWidget(QStringList playersList, QWidget *parent)
    : QWidget(parent)
{
  playersGamingOrder = playersList;
  textWidth = 0;
  playersOrder = "";
  firstPosition = 1;
  numberOfPlayers = playersList.count();
  setMatchesDone(false);

  int numberOfMatches = 0;

  if(numberOfPlayers == 2) {
    rows = 4;
    rowsOffset = -1;
    numberOfMatches = 1;
    rowSequence[0] << 1 << 3;
    rowSequence[1] << 2;
  } else if(numberOfPlayers == 4) {
    rows = 9;
    rowsOffset = 0;
    numberOfMatches = 4;
    rowSequence[0] << 1 << 3 << 5 << 7;
    rowSequence[1] << 2 << 6;
    rowSequence[2] << 4;
  } else if(numberOfPlayers == 8) {
    rows = 19;
    rowsOffset = 2;
    numberOfMatches = 10;
    rowSequence[0] << 1 << 3 << 5 << 7 << 9 << 11 << 13 << 15;
    rowSequence[1] << 2 << 6 << 10 << 14;
    rowSequence[2] << 4 << 12;
    rowSequence[3] << 8;
  }

  countPerSide = 0;
  for(int i = 1; i < rowSequence.count(); i++) {
    countPerSide += rowSequence[i].count();
  }
  
  lblOrder = new QLabel(this);
  lblOrder->setText(playersOrder);

  font = lblOrder->font();
  QFontMetrics fm(font);

  for(int i = 0; i < numberOfPlayers; i++) {
    if(textWidth < fm.width(playersGamingOrder.at(i)))
    textWidth = fm.width(playersGamingOrder.at(i));
  }

  for(int i = numberOfPlayers; i < rows + 1 + rowsOffset; i++)
    playersGamingOrder << "";

  
  for(int i = 0; i < numberOfMatches; i++) {
    ledMatchSet << new QLineEdit(this);
    ledMatchSet[i]->setAlignment(Qt::AlignCenter);
    ledMatchSet[i]->resize(50, 20);
    ledMatchSet[i]->setObjectName(QString("%1").arg(i));
    ledMatchSet[i]->setInputMask("9:9");
    
    connect(ledMatchSet[i], SIGNAL(editingFinished()),
            this, SLOT(editingFinished()));
  }

  int width = ((rowSequence.count() - 1) * 2 + 1) * (textWidth + 2 * rowTextOffset + 2 * rowOffset);
  resize(width, width / 2);
  setMinimumSize(width, width / 2);
}

bool ZSpiderWidget::matchesDone()
{
  return _matchesDone;
}

void ZSpiderWidget::setMatchesDone(bool done)
{
  _matchesDone = done;
}

int ZSpiderWidget::position()
{
  return firstPosition;
}

void ZSpiderWidget::setPosition(int pos)
{
  firstPosition = pos;
}

QStringList ZSpiderWidget::results()
{
  QStringList list;
  for(int i = 0; i < ledMatchSet.count(); i++)
    list << ledMatchSet[i]->text();
  
  return list;
}

void ZSpiderWidget::setResults(QStringList list)
{
  for(int i = 0; i < ledMatchSet.count(); i++)
    ledMatchSet[i]->setText(list.at(i));
}

void ZSpiderWidget::editingFinished()
{
  QLineEdit *led = (QLineEdit *) sender();
  QStringList match = led->text().simplified().replace(" ","").split(":");
  QString name = led->objectName();
  int num = name.toUInt();

  spiderUpdate(num);

  repaint();
  emit windowModified(true);
}

void ZSpiderWidget::spiderUpdateAll()
{
  for(int i = 0; i < ledMatchSet.count(); i++)
    spiderUpdate(i);

  repaint();
}

void ZSpiderWidget::spiderUpdate(int num)
{
  int step = (num % 2) ? 1 : 0;
  QStringList matchStringList = ledMatchSet[num]->text().simplified().replace(" ","").split(":");

  QList<int> match;
  for(int i = 0; i < matchStringList.count(); i++)
    match << matchStringList.at(i).toUInt();

  if(num < numberOfPlayers / 2) {
    step = num;
    if(match[0] > match[1]) {
      playersGamingOrder[numberOfPlayers + num] = playersGamingOrder[num + step];
      playersGamingOrder[numberOfPlayers * 3 / 2 + num] = playersGamingOrder[num + step + 1];
    } else if(match[0] < match[1]) {
      playersGamingOrder[numberOfPlayers + num] = playersGamingOrder[num + step + 1];
      playersGamingOrder[numberOfPlayers * 3 / 2 + num] = playersGamingOrder[num + step];
    } else {
      playersGamingOrder[numberOfPlayers + num] = "";
      playersGamingOrder[numberOfPlayers * 3 / 2 + num] = "";
    }
  } else if(num >= numberOfPlayers / 2 && num < numberOfPlayers * 3 / 4) {
    step = num;
    int step2 = num % (numberOfPlayers / 2) ? num -1: num;
    if(match[0] > match[1]) {
      playersGamingOrder[numberOfPlayers + num + step2] = playersGamingOrder[num + step];
    } else if(match[0] < match[1]) {
      playersGamingOrder[numberOfPlayers + num + step2] = playersGamingOrder[num + step + 1];
    } else {
      playersGamingOrder[numberOfPlayers + num + step2] = "";
    }
  } else if(num >= numberOfPlayers * 3 / 4 && num < numberOfPlayers) {
    step = num;
    int step2 = 0;
    if(num == numberOfPlayers * 3 / 4) {
      if(numberOfPlayers == 4)
        step2 = num -1;
      else
        step2 = num -2;
    } else if(num > numberOfPlayers * 3 / 4)
      step2 = num -3;
    if(match[0] > match[1]) {
      playersGamingOrder[numberOfPlayers + num + step2] = playersGamingOrder[num + step];
    } else if(match[0] < match[1]) {
      playersGamingOrder[numberOfPlayers + num + step2] = playersGamingOrder[num + step + 1];
    } else {
      playersGamingOrder[numberOfPlayers + num + step2] = "";
    }
  } else if(num >= numberOfPlayers && num < numberOfPlayers * 5 / 4) {
    step = num;
    int step2 = step/2;
    if(match[0] > match[1]) {
      playersGamingOrder[numberOfPlayers + num + step2] = playersGamingOrder[num + step];
    } else if(match[0] < match[1]) {
      playersGamingOrder[numberOfPlayers + num + step2] = playersGamingOrder[num + step + 1];
    } else {
      playersGamingOrder[numberOfPlayers + num + step2] = "";
    }

  }

  bool allMatchesCompleted = true;
  for(int i = 0; i < ledMatchSet.count(); i++) {
    if(ledMatchSet[i]->text() == ":")
      allMatchesCompleted = false;
  }

  if(allMatchesCompleted) {
    QStringList orderList;
    playersOrderFinal.clear();

    if(numberOfPlayers == 2) {
      int max = rows + rowsOffset + 1;
      for(int i = max - (numberOfPlayers * 3 / 2 + rowsOffset); i < max; i++)
        orderList << playersGamingOrder.at(i);
    
      if(ledMatchSet[0]->text().split(":").at(0).toUInt() > ledMatchSet[0]->text().split(":").at(1).toUInt()) {
        playersOrderFinal[0] = orderList.at(0);
        playersOrderFinal[1] = orderList.at(1);
      } else if(ledMatchSet[0]->text().split(":").at(0).toUInt() < ledMatchSet[0]->text().split(":").at(1).toUInt()) {
        playersOrderFinal[1] = orderList.at(1);
        playersOrderFinal[0] = orderList.at(0);
      }
    } else if(numberOfPlayers == 4) {
      int max = rows + rowsOffset + 1;
      for(int i = max - (numberOfPlayers * 3 / 2 + rowsOffset); i < max; i++)
        orderList << playersGamingOrder.at(i);
      
      int step = numberOfPlayers / 2;
      for(int i = 1; i < step + 1; i++) {
        playersOrderFinal[numberOfPlayers - (i * step - 1) - 1] = playersGamingOrder.at(playersGamingOrder.count() - i);
        orderList.removeAll(playersGamingOrder.at(playersGamingOrder.count() - i));
      }

      for(int i = 0; i < step; i++)
        playersOrderFinal[(i * step + step) - 1] = orderList.at(i);

    } else if(numberOfPlayers == 8) {
      int max = rows + rowsOffset + 1;
      for(int i = max - (numberOfPlayers * 3 /2 + rowsOffset); i < max; i++)
        orderList << playersGamingOrder.at(i);

      int step = numberOfPlayers / 2;
      int step2 = numberOfPlayers / 4;
      for(int i = 1; i < step2 + 1; i++) {
        playersOrderFinal[numberOfPlayers - (i * step - 1) - 1] = playersGamingOrder.at(playersGamingOrder.count() - i);
        orderList.removeLast();
      }

      QStringList orderListSec;
      for(int i = max - (step + rowsOffset); i < max - step2; i++)
        orderListSec << playersGamingOrder.at(i);

      for(QMap<int, QString>::iterator iterator = playersOrderFinal.begin(); iterator != playersOrderFinal.end(); ++iterator ) {
        orderListSec.removeAll(iterator.value());
      }

      for(int i = 0; i < 2; i++)
        playersOrderFinal[(2 + i * step) - 1] = orderListSec.at(i);
      
      orderListSec.clear();
      for(int i = max - (numberOfPlayers * 3 / 2 + rowsOffset); i < max - (step + step2); i++)
        orderListSec << playersGamingOrder.at(i);

      for(QMap<int, QString>::iterator iterator = playersOrderFinal.begin(); iterator != playersOrderFinal.end(); ++iterator ) {
        orderListSec.removeAll(iterator.value());
      }

      for(int i = 0; i < 4; i += 2) {
        int pos = !i ? 2 : 6;
        playersOrderFinal[i % 2 + pos] = orderListSec.at(i);
        playersOrderFinal[i % 2 + 1 + pos] = orderListSec.at(i+1);
      }
    }

    playersOrder = "";
    for(QMap<int, QString>::iterator iterator = playersOrderFinal.begin(); iterator != playersOrderFinal.end(); ++iterator ) {
      playersOrder += QString("%1. %2").arg(firstPosition + iterator.key()).arg(iterator.value());
      if(iterator != playersOrderFinal.end() - 1)
        playersOrder += ", ";
    }
    lblOrder->setText(playersOrder);

    setMatchesDone(true);
    emit matchDone(firstPosition, playersOrderFinal);
  }
}

void ZSpiderWidget::paintEvent(QPaintEvent *e)
{
  Q_UNUSED(e);  
  
  QPainter painter(this);

  painter.setFont(font);
  QFontMetrics fm(font);

  int height = this->height();
  int width = this->width();


  lblOrder->move(rowOffset, height - rowOffset - rowWidthOffset);
  lblOrder->resize(width, lblOrder->height());
  
  int ledMatchSetIndex = 0;
  int playersGamingOrderIndex = 0;
  QPainterPath spiderPath;
  for(int j = 0; j < rowSequence.count(); j++) {
    for(int k = 0; k < 2; k++) {
      for(int i = 0; i < rowSequence[j].count(); i++) {
        double row = rowSequence[j][i];
        int sider = qPow(-1, k);
        int widthOffset = width / 2 + (rowWidthOffset * 2 + textWidth) * j * sider;
        int heightOffset = height / rows * row + rowOffset;

        if((!j && !k) || j) {
          int textWidthForIdx = fm.width(playersGamingOrder.at(playersGamingOrderIndex));
          painter.drawText(widthOffset - textWidthForIdx / 2, heightOffset - rowTextOffset, playersGamingOrder.at(playersGamingOrderIndex++));
          spiderPath.moveTo(widthOffset - rowWidthOffset - textWidth / 2, heightOffset);
          spiderPath.lineTo(widthOffset + rowWidthOffset + textWidth / 2, heightOffset);

          if(j < rowSequence.count() - 1 && !(i % 2) && rowSequence[j].count() > 1) {
            int heightOffsetLocal = (height / rows * rowSequence[j][i+1] + rowOffset - heightOffset) / 2 + heightOffset;
            int ledWidth = ledMatchSet[ledMatchSetIndex]->width() / 2;
            int ledHeight = ledMatchSet[ledMatchSetIndex]->height() / 2;
            ledMatchSet[ledMatchSetIndex]->move(widthOffset - ledWidth, heightOffsetLocal - rowTextOffset - ledHeight);
            ledMatchSetIndex++;
          }
        }

        if(!(i % 2) && rowSequence[j].count() > 1) {
          int widthOffsetLocal = widthOffset + (rowWidthOffset + textWidth / 2) * sider;
          spiderPath.moveTo(widthOffsetLocal, heightOffset);
          spiderPath.lineTo(widthOffsetLocal, height / rows * rowSequence[j][i+1] + rowOffset); 
        }
      }
    }
  }
  spiderPath.moveTo(0, height - 1);
  spiderPath.lineTo(width, height - 1);
  spiderPath.closeSubpath();

  painter.drawPath(spiderPath);

}

