#include <QtGui>
#include "toma.h"
#include "zteammatchesdelegate.h"


ToMa *toma;
QDialog *editor;
QTableView *tbvTeamMatches;
QAbstractItemModel *modit;

int fixedHeight = 125;

ZTeamMatchesDelegate::ZTeamMatchesDelegate(QObject *parent, int team)
    : QItemDelegate(parent)
{
  this->team = team;
  toma = static_cast<ToMa*>(parent);
}

QWidget *ZTeamMatchesDelegate::createEditor(QWidget *parent,
    const QStyleOptionViewItem &/* option */,
    const QModelIndex &/* index */) const
{
  Q_UNUSED(parent);

  editor = new QDialog();

  tbvTeamMatches = new QTableView(editor);
  tbvTeamMatches->setSelectionMode(QAbstractItemView::SingleSelection);
  
  QPushButton *btnOk = new QPushButton(tr("&Ok"), editor);
  QPushButton *btnAddMatch = new QPushButton(tr("&Add match"), editor);
  btnOk->setMaximumWidth(200);
  btnAddMatch->setMaximumWidth(200);

  tbvTeamMatches->resize(350, 60);
  tbvTeamMatches->setGeometry(5, 5, 350, 60);

  QHBoxLayout *hblButtons = new QHBoxLayout;
  hblButtons->addWidget(btnOk, 0, Qt::AlignLeft);
  hblButtons->addWidget(btnAddMatch, 1, Qt::AlignLeft);

  QVBoxLayout *vblMain = new QVBoxLayout;
  vblMain->addWidget(tbvTeamMatches);
  vblMain->addLayout(hblButtons);

  editor->setLayout(vblMain);
  
  connect(btnOk, SIGNAL(clicked()),
          this, SLOT(okClicked()));
  connect(btnAddMatch, SIGNAL(clicked()),
          this, SLOT(addClicked()));
  
  editor->setWindowTitle(toma->appName() + " - " + tr("Group Players Match"));
  editor->resize(360, fixedHeight);  
  editor->setMinimumSize(360, fixedHeight);  
  editor->setFixedHeight(fixedHeight);
  editor->setWindowModality(Qt::ApplicationModal);
  editor->setModal(true);

  return editor;
}

void ZTeamMatchesDelegate::setEditorData(QWidget *editor,
                                    const QModelIndex &index) const
{
  editor = static_cast<QDialog*>(editor);

  modit = toma->teamPlayersModel(team, index.row(), index.column());
  modit->setHeaderData(modit->columnCount() - 1, Qt::Horizontal, tr("Sets")); 
  for (int row = 0; row < modit->rowCount(); ++row)
    for (int column = 0; column < modit->columnCount(); ++column)
      modit->setData(modit->index(row, column, QModelIndex()), Qt::AlignCenter, Qt::TextAlignmentRole);

  tbvTeamMatches->setModel(modit);
  tbvTeamMatches->resizeRowsToContents();
  tbvTeamMatches->resizeColumnsToContents();

  adjustContentSize();
  editor->move((QApplication::desktop()->width() - editor->width())/2,
               (QApplication::desktop()->height() - editor->height())/2);

  connect(toma->teamPlayersModel(team, index.row(), index.column()), SIGNAL(itemChanged(QStandardItem*)),
          this, SLOT(itemChange(QStandardItem*)));
}

void ZTeamMatchesDelegate::setModelData(QWidget *editor, QAbstractItemModel *model,
                                   const QModelIndex &index) const
{
  Q_UNUSED(editor);

  QAbstractItemModel *modit = tbvTeamMatches->model();
  QModelIndex indexInv = model->index(index.column(), index.row());

  QString sets = modit->data(modit->index(0, modit->columnCount() - 1), Qt::DisplayRole).toString() + ":" + modit->data(modit->index(1, modit->columnCount() - 1), Qt::DisplayRole).toString();
  QString setsA = modit->data(modit->index(0, modit->columnCount() - 1), Qt::DisplayRole).toString();
  QString setsB = modit->data(modit->index(1, modit->columnCount() - 1), Qt::DisplayRole).toString();

  if(!setsA.isEmpty() && !setsB.isEmpty()) {
    model->setData(index, setsA + ":" + setsB, Qt::EditRole);
    model->setData(indexInv, setsB + ":" + setsA, Qt::EditRole);
  }
}

void ZTeamMatchesDelegate::itemChange(QStandardItem *item)
{
  Q_UNUSED(item);
  if(item->column() != modit->columnCount() - 1) {  
    quint8 pointsA = 0;
    quint8 pointsB = 0;
    for(int column = 0; column < modit->columnCount() - 1; column++) {
      quint8 a = modit->data(modit->index(0, column), Qt::DisplayRole).toUInt();
      quint8 b = modit->data(modit->index(1, column), Qt::DisplayRole).toUInt();
      if(a - b >= 2)
        pointsA++;
      else if(b - a >= 2)
        pointsB++;
    }
    modit->setData(modit->index(0, modit->columnCount() - 1), pointsA, Qt::EditRole);
    modit->setData(modit->index(1, modit->columnCount() - 1), pointsB, Qt::EditRole);
    tbvTeamMatches->resizeColumnsToContents();
  }
}

void ZTeamMatchesDelegate::okClicked()
{
  editor->close();
}

void ZTeamMatchesDelegate::addClicked()
{
  tbvTeamMatches->model()->insertColumn(tbvTeamMatches->model()->columnCount() - 1);
  adjustContentSize();
}

void ZTeamMatchesDelegate::adjustContentSize() const
{
  tbvTeamMatches->resizeColumnsToContents();
  tbvTeamMatches->resizeRowsToContents();
  int colSize = tbvTeamMatches->verticalHeader()->size().width();
  for(int i = 0; i < tbvTeamMatches->model()->columnCount(); i++)
    colSize += tbvTeamMatches->columnWidth(i);
  tbvTeamMatches->setMinimumWidth(colSize + 5);
  editor->resize(colSize + 26, fixedHeight);  
}

QSize ZTeamMatchesDelegate::sizeHint(const QStyleOptionViewItem &option,
                               const QModelIndex &index) const
{
  QStyledItemDelegate *sid = new QStyledItemDelegate();
  QSize size = sid->sizeHint(option, index);
  size.setWidth(30);
  return size;
}

